# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# When tested locally, please use 'azdev test cloud -a "-n 1" to run in serial'

from azure.cli.testsdk import ScenarioTest
from azure.cli.testsdk.decorators import serial_test


class CloudTests(ScenarioTest):
    def setUp(self):
        self.cloudname = self.cmd('az cloud show').get_output_in_json().get('name')

    def tearDown(self):
        # cli_ctx cloud name is used to check with the cloud name to set
        # If both are same, it will skip the switch
        # The test case cli_ctx isn't change even when the cloud is really changed
        # So here set the cli_ctx cloud name to empty to enforce cloud to switch
        self.cli_ctx.cloud.name = ''
        self.cmd('az cloud set -n ' + self.cloudname)

    @serial_test()
    def test_cloud_set_AzureCloud(self):
        self.cmd('az cloud set -n AzureCloud')
        self.cmd('az cloud show -n AzureCloud', checks=[self.check('isActive', True)])

    @serial_test()
    def test_cloud_set_AzureChinaCloud(self):
        self.cmd('az cloud set -n AzureChinaCloud')
        self.cmd('az cloud show -n AzureChinaCloud', checks=[self.check('isActive', True)])

    @serial_test()
    def test_cloud_set_AzureUSGovernment(self):
        self.cmd('az cloud set -n AzureUSGovernment')
        self.cmd('az cloud show -n AzureUSGovernment', checks=[self.check('isActive', True)])

    @serial_test()
    def test_cloud_set_AzureGermanCloud(self):
        self.cmd('az cloud set -n AzureGermanCloud')
        self.cmd('az cloud show -n AzureGermanCloud', checks=[self.check('isActive', True)])

    @serial_test()
    def test_cloud_set_azurecloud(self):
        self.cmd('az cloud set -n azurecloud')
        self.cmd('az cloud show -n AzureCloud', checks=[self.check('isActive', True)])

    @serial_test()
    def test_cloud_set_azurechinacloud(self):
        self.cmd('az cloud set -n azurechinacloud')
        self.cmd('az cloud show -n AzureChinaCloud', checks=[self.check('isActive', True)])

    @serial_test()
    def test_cloud_set_azureusgovernment(self):
        self.cmd('az cloud set -n azureusgovernment')
        self.cmd('az cloud show -n AzureUSGovernment', checks=[self.check('isActive', True)])

    @serial_test()
    def test_cloud_set_azuregermancloud(self):
        self.cmd('az cloud set -n azuregermancloud')
        self.cmd('az cloud show -n AzureGermanCloud', checks=[self.check('isActive', True)])

    @serial_test()
    def test_cloud_set_unregistered_cloud_name(self):
        self.cmd('az cloud set -n azCloud', expect_failure=True)


class SubscriptionSuppressionTest(ScenarioTest):

    def test_subscription_suppression(self):
        from knack.util import CLIError
        self.cmd('az cloud list')

        # this should fail with an "unrecognized argument" error
        with self.assertRaisesRegexp(SystemExit, '2'):
            self.cmd('az cloud list --subscription foo')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# Azure Killer
# version 0.1
# Clean Azure resources automatically
# Feiyue Yu

import os
import json
import datetime

# Please set parameters

# Subscription ID
subscription = '0b1f6471-1bf0-4dda-aec3-cb9272f09590'

# Prefix of resource group that will be deleted
prefixes = ['cli_test', 'clitest']

# Maximum survival time, in days
TTL = 1


def main():
    print('Azure Killer, version 0.1')
    print('Configuration:')
    print('    Subscription: ' + subscription)
    # print('    Resource group prefix: ' + str(prefixes))
    # print('    Maximum survival time: %d days' % TTL)
    print()
    cmd = 'az group list --subscription %s --query [].name'
    result = os.popen(cmd % subscription).read()
    rgs = json.loads(result)
    for rg in rgs:
        clean_rg(rg)


def clean_rg(rg):
    """
    Clean resource group.
    :param rg: Resource group name
    :return:
    """
    print('Processing resource group: ' + rg)
    cmd = 'az group delete -y -g %s --subscription %s' % (rg, subscription)
    print(cmd)
    os.popen(cmd)


def old_enough(dates):
    """
    Whether it's old enough.
    :param dates: Array of dates
    :return: bool
    """
    if not dates:
        print('Duration: too old')
        return True
    date = dates[-1]
    date = datetime.datetime.strptime(date, '%Y-%m-%dT%H:%M:%S.%f+00:00')
    now = datetime.datetime.utcnow()
    duration = now - date
    print('Duration: ' + str(duration))
    return duration.days > TTL


def target_rg(rg):
    """
    Whether rg has certain prefix.
    :param rg: Resource group name
    :return: bool
    """
    return any(rg.startswith(prefix) for prefix in prefixes)


if __name__ == '__main__':
    main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import os
import json
import traceback


class TestData:
    """
    Model for testing results
    """
    def __init__(self, artifact_dir):
        # Value of the list should be (module_name, passed, failed, pass_rate)
        self.modules = []
        # ('Total', passed, failed, pass_rate)
        self.total = None
        self.artifact_dir = artifact_dir

    def collect(self):
        """
        Collect data
        :return:
        """
        print('Generating summary...')
        data_dict = {}
        for root, dirs, files in os.walk(self.artifact_dir):
            for name in files:
                if name.endswith('json'):
                    try:
                        print('Reading {}'.format(name))
                        module = name.split('.')[0]
                        with open(os.path.join(root, name)) as f:
                            result = json.loads(f.read())
                            passed = failed = 0
                            if 'passed' in result['summary']:
                                passed = result['summary']['passed']
                            if 'failed' in result['summary']:
                                failed = result['summary']['failed']
                            if module in data_dict:
                                values = data_dict[module]
                                data_dict[module] = (values[0] + passed, values[1] + failed)
                            else:
                                data_dict[module] = (passed, failed)
                    except Exception:
                        print(traceback.format_exc())

        passed_sum = failed_sum = 0
        for k in data_dict:
            v = data_dict[k]
            passed = v[0]
            failed = v[1]
            total = passed + failed
            rate = 1 if total == 0 else passed / total
            rate = '{:.2%}'.format(rate)
            self.modules.append((k, passed, failed, rate))
            print('module: {}, passed: {}, failed: {}, rate: {}'.format(k, passed, failed, rate))
            passed_sum += passed
            failed_sum += failed

        print(self.modules)
        print('Sorting...')
        sorted_modules = sorted(self.modules, key=lambda x: x[0])
        self.modules = sorted_modules
        print(self.modules)

        total_sum = passed_sum + failed_sum
        rate_sum = 1 if total_sum == 0 else passed_sum / total_sum
        rate_sum = '{:.2%}'.format(rate_sum)
        self.total = ('Total', passed_sum, failed_sum, rate_sum)
        print('module: Total, passed: {}, failed: {}, rate: {}'.format(passed_sum, failed_sum, rate_sum))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


import sys
import os
import yaml
from knack.help_files import helps
from .linter import LinterManager


def define_arguments(parser):
    parser.add_argument('--ci', action='store_true', help='Run in CI mode')
    parser.add_argument('--params', dest='rule_types_to_run', action='append_const', const='params',
                        help='Run linter on parameters.')
    parser.add_argument('--commands', dest='rule_types_to_run', action='append_const', const='commands',
                        help='Run linter on commands.')
    parser.add_argument('--command-groups', dest='rule_types_to_run', action='append_const', const='command_groups',
                        help='Run linter on command groups.')
    parser.add_argument('--help-file-entries', dest='rule_types_to_run', action='append_const', const='help_entries',
                        help='Run linter on help-file-entries.')
    parser.add_argument('--modules', dest='modules', nargs='+', help='The modules on which the linter should run.')
    parser.add_argument('--extensions', dest='extensions', nargs='+', help='The extensions on which the linter should run.')
    parser.add_argument('--rules', dest='rules', nargs='+', help='The rules which the linter should run.')


def main(args):
    from azure.cli.core import get_default_cli
    from azure.cli.core.file_util import get_all_help, create_invoker_and_load_cmds_and_args

    print('Initializing linter with command table and help files...')
    # setup CLI to enable command loader
    az_cli = get_default_cli()

    # load commands, args, and help
    create_invoker_and_load_cmds_and_args(az_cli)
    loaded_help = get_all_help(az_cli)
    command_loader = az_cli.invocation.commands_loader

    # format loaded help
    loaded_help = {data.command: data for data in loaded_help if data.command}

    # load yaml help
    help_file_entries = {}
    for entry_name, help_yaml in helps.items():
        help_entry = yaml.safe_load(help_yaml)
        help_file_entries[entry_name] = help_entry

    if not args.rule_types_to_run:
        args.rule_types_to_run = ['params', 'commands', 'command_groups', 'help_entries']

    # find rule exclusions and pass to linter manager
    from ..utilities.path import get_command_modules_paths, get_extensions_paths
    exclusions = {}
    command_modules_paths = get_command_modules_paths()
    extension_paths = get_extensions_paths()
    for gen in (command_modules_paths, extension_paths):
        for _, path in gen:
            exclusion_path = os.path.join(path, 'linter_exclusions.yml')
            if os.path.isfile(exclusion_path):
                mod_exclusions = yaml.safe_load(open(exclusion_path))
                exclusions.update(mod_exclusions)

    # only run linter on modules and extensions specified
    if args.modules or args.extensions:
        from .util import include_commands
        command_loader, help_file_entries = include_commands(
            command_loader, help_file_entries, module_inclusions=args.modules, extensions=args.extensions)

    # Instantiate and run Linter
    linter_manager = LinterManager(command_loader=command_loader,
                                   help_file_entries=help_file_entries,
                                   loaded_help=loaded_help,
                                   exclusions=exclusions,
                                   rule_inclusions=args.rules)
    exit_code = linter_manager.run(run_params='params' in args.rule_types_to_run,
                                   run_commands='commands' in args.rule_types_to_run,
                                   run_command_groups='command_groups' in args.rule_types_to_run,
                                   run_help_files_entries='help_entries' in args.rule_types_to_run,
                                   ci=args.ci)

    sys.exit(exit_code)


def init_args(root):
    parser = root.add_parser('cli-lint', help="Verify the contents of the command table and help.")
    define_arguments(parser)
    parser.set_defaults(func=main)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azure.cli.testsdk import ResourceGroupPreparer, JMESPathCheck
from azure.cli.testsdk import ScenarioTest
from azure.mgmt.cdn.models import SkuName
from .scenario_mixin import CdnScenarioMixin


class CdnProfileScenarioTest(CdnScenarioMixin, ScenarioTest):
    @ResourceGroupPreparer()
    def test_cdn_profile_crud(self, resource_group):
        list_checks = [JMESPathCheck('length(@)', 0)]
        self.profile_list_cmd(resource_group, checks=list_checks)

        profile_name = 'profile123'
        checks = [JMESPathCheck('name', profile_name),
                  JMESPathCheck('sku.name', SkuName.standard_akamai.value)]
        self.profile_create_cmd(resource_group, profile_name, checks=checks)
        self.profile_show_cmd(resource_group, profile_name, checks=checks)

        list_checks = [JMESPathCheck('length(@)', 1)]
        self.profile_list_cmd(resource_group, checks=list_checks)

        checks.append(JMESPathCheck('tags.foo', 'bar'))
        self.profile_update_cmd(resource_group, profile_name, tags='foo=bar', checks=checks)

        self.profile_delete_cmd(resource_group, profile_name)


class CdnProfileDeleteDoesNotExistScenarioTest(CdnScenarioMixin, ScenarioTest):
    @ResourceGroupPreparer()
    def test_cdn_profile_delete_not_found(self, resource_group):
        # see https://github.com/Azure/azure-cli/issues/3304
        # request should respond with a 204 rather than a 404
        res = self.profile_delete_cmd(resource_group, "foo12345")
        self.assertEqual("", res.output)


class CdnProfileMicrosoftScenarioTest(CdnScenarioMixin, ScenarioTest):
    @ResourceGroupPreparer()
    def test_cdn_microsoft_standard_sku(self, resource_group):
        # https://github.com/Azure/azure-cli/issues/7635
        self.kwargs.update({
            'profile': 'cdnprofile1',
            'sku': 'Standard_Microsoft',
            'rg': resource_group,
        })
        self.cmd('cdn profile create -g {rg} -n {profile} --sku {sku}')



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------
import inspect
import logging
import os
import sys
import traceback
import datetime as dt

from azure.core.exceptions import AzureError
from azure.cli.testsdk.exceptions import CliTestError, CliExecutionError, JMESPathCheckAssertionError


logger = logging.getLogger('azure.cli.testsdk')
logger.addHandler(logging.StreamHandler())
__path__ = __import__('pkgutil').extend_path(__path__, __name__)
exceptions = []
test_map = dict()
SUCCESSED = "successed"
FAILED = "failed"


def try_manual(func):
    def import_manual_function(origin_func):
        from importlib import import_module
        decorated_path = inspect.getfile(origin_func).lower()
        module_path = __path__[0].lower()
        if not decorated_path.startswith(module_path):
            raise Exception("Decorator can only be used in submodules!")
        manual_path = os.path.join(
            decorated_path[module_path.rfind(os.path.sep) + 1:])
        manual_file_path, manual_file_name = os.path.split(manual_path)
        module_name, _ = os.path.splitext(manual_file_name)
        manual_module = "..manual." + \
            ".".join(manual_file_path.split(os.path.sep) + [module_name, ])
        return getattr(import_module(manual_module, package=__name__), origin_func.__name__)

    def get_func_to_call():
        func_to_call = func
        try:
            func_to_call = import_manual_function(func)
            logger.info("Found manual override for %s(...)", func.__name__)
        except (ImportError, AttributeError):
            pass
        return func_to_call

    def wrapper(*args, **kwargs):
        func_to_call = get_func_to_call()
        logger.info("running %s()...", func.__name__)
        try:
            test_map[func.__name__] = dict()
            test_map[func.__name__]["result"] = SUCCESSED
            test_map[func.__name__]["error_message"] = ""
            test_map[func.__name__]["error_stack"] = ""
            test_map[func.__name__]["error_normalized"] = ""
            test_map[func.__name__]["start_dt"] = dt.datetime.utcnow()
            ret = func_to_call(*args, **kwargs)
        except (AssertionError, AzureError, CliTestError, CliExecutionError, SystemExit,
                JMESPathCheckAssertionError) as e:
            use_exception_cache = os.getenv("TEST_EXCEPTION_CACHE")
            if use_exception_cache is None or use_exception_cache.lower() != "true":
                raise
            test_map[func.__name__]["end_dt"] = dt.datetime.utcnow()
            test_map[func.__name__]["result"] = FAILED
            test_map[func.__name__]["error_message"] = str(e).replace("\r\n", " ").replace("\n", " ")[:500]
            test_map[func.__name__]["error_stack"] = traceback.format_exc().replace(
                "\r\n", " ").replace("\n", " ")[:500]
            logger.info("--------------------------------------")
            logger.info("step exception: %s", e)
            logger.error("--------------------------------------")
            logger.error("step exception in %s: %s", func.__name__, e)
            logger.info(traceback.format_exc())
            exceptions.append((func.__name__, sys.exc_info()))
        else:
            test_map[func.__name__]["end_dt"] = dt.datetime.utcnow()
            return ret

    if inspect.isclass(func):
        return get_func_to_call()
    return wrapper


def calc_coverage(filename):
    filename = filename.split(".")[0]
    coverage_name = filename + "_coverage.md"
    with open(coverage_name, "w") as f:
        f.write("|Scenario|Result|ErrorMessage|ErrorStack|ErrorNormalized|StartDt|EndDt|\n")
        total = len(test_map)
        covered = 0
        for k, v in test_map.items():
            if not k.startswith("step_"):
                total -= 1
                continue
            if v["result"] == SUCCESSED:
                covered += 1
            f.write("|{step_name}|{result}|{error_message}|{error_stack}|{error_normalized}|{start_dt}|"
                    "{end_dt}|\n".format(step_name=k, **v))
        f.write("Coverage: {}/{}\n".format(covered, total))
    print("Create coverage\n", file=sys.stderr)


def raise_if():
    if exceptions:
        if len(exceptions) <= 1:
            raise exceptions[0][1][1]
        message = "{}\nFollowed with exceptions in other steps:\n".format(str(exceptions[0][1][1]))
        message += "\n".join(["{}: {}".format(h[0], h[1][1]) for h in exceptions[1:]])
        raise exceptions[0][1][0](message).with_traceback(exceptions[0][1][2])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import sys
import os
import subprocess


def get_repo_root():
    """
    Returns the root path to this repository. The root is where .git folder is.
    """
    import os.path
    here = os.path.dirname(os.path.realpath(__file__))

    while not os.path.exists(os.path.join(here, '.git')):
        here = os.path.dirname(here)

    return here


def comment_import_help(init_file, out_file):
    f_out =  open(out_file, "w")

    output = ""
    updated = False
    with open(init_file, "r") as f_in:
        for line in f_in:
            if "import" in line and "_help" in line and not updated:
                updated = True
                line = "# " + line
            output += line

    f_out.write(output)
    f_out.close()
    return updated

def decomment_import_help(init_file, out_file):
    f_out =  open(out_file, "w")

    output = ""
    updated = False
    with open(init_file, "r") as f_in:
        for line in f_in:
            if "import" in line and "_help" in line and not updated:
                updated = True
                line = line.lstrip("# ")
            output += line
    f_out.write(output)
    f_out.close()
    return updated

def install_extension(ext_name):
    command = "az extension add -n " + ext_name
    completed = subprocess.run(command.split())
    if completed.returncode == 0:
        print("{} was successfully installed.".format(ext_name))
        return True
    else:
        print("{} was not installed.".format(ext_name))
        return False

def uninstall_extension(ext_name):
    command = "az extension remove -n " + ext_name
    completed = subprocess.run(command.split())
    if completed.returncode == 0:
        print("{} was successfully uninstalled.".format(ext_name))
        return True
    else:
        print("{} was not uninstalled.".format(ext_name))
        return False

if __name__ == "__main__":
    args = sys.argv[1:]

    if args:
        if args[0].lower() == "--core":
            test = False
            try:
                if args[1].lower() == "--test":
                    test = True
            except IndexError:
                pass

            subprocess.run(["python", "./help_convert.py", "--get-all-mods"])

            module_names = []
            with open("mod.txt", "r") as f:
                for line in f:
                    module_names.append(line)
            if "sqlvm" not in module_names:
                module_names.append("sqlvm")
            os.remove("mod.txt")
            successes = 0
            with open(os.devnull, 'w') as devnull: # silence stdout by redirecting to devnull
                for mod in module_names:
                    args = ["python", "./help_convert.py", mod]
                    if test:
                        args.append("--test")
                    completed_process = subprocess.run(args, stdout=devnull)
                    if completed_process.returncode == 0:
                        successes += 1

            if successes:
                print("\n----------------------------------------------------------"
                      "\nSuccessfuly converted {} help.py files to help.yaml files."
                      "\n----------------------------------------------------------".format(successes))

        elif args[0].lower() == "--extensions":
            pass

        elif args[0].lower() == "--count":
            # Get info about help.py modules and converted help.yaml modules.

            src_root = os.path.join(get_repo_root(), "src", "command_modules")

            py_count = 0
            yaml_count = 0

            for root, dirs, files in os.walk(src_root):
                for file in files:
                    if file.endswith("_help.py") and os.path.dirname(root).endswith("command_modules") and os.path.join("build", "lib") not in root:
                        print("Found {}\n".format(os.path.join(root, file)))
                        py_count +=1
                    if file.endswith("help.yaml") and os.path.dirname(root).endswith("command_modules"):
                        print("Found {}\n".format(os.path.join(root, file)))
                        yaml_count +=1

            print("Found {} _help.py files\n".format(py_count))
            print("Found {} help.yaml files\n".format(yaml_count))

        elif args[0].lower() == "--move-py":

            src_root = os.path.join(get_repo_root(), "src", "command_modules")

            py_count = 0
            yaml_count = 0
            failures = 0

            for root, dirs, files in os.walk(src_root):
                for file in files:
                    if file.endswith("_help.py") and os.path.dirname(root).endswith("command_modules") and os.path.join("build", "lib") not in root:
                        src = os.path.join(root, file)
                        dst = os.path.join(root, "foo.py")
                        print("Found {}\n".format(src))
                        print("Renaming {}\n\tto {}\n.".format(src, dst))
                        os.rename(src, dst)
                        py_count +=1

                        src = os.path.join(root, "__init__.py")
                        dst = os.path.join(root, "__init__2.py")

                        success = comment_import_help(src, dst)

                        if success:
                            os.remove(src)
                            os.rename(dst, src)
                            print("Commented out import in {}\n".format(src))
                        else:
                            os.remove(dst)
                            print("Failed to comment out import in {}\n".format(src))
                            failures+=1


            print("Renamed {} _help.py files to foo.py.\n".format(py_count))
            print("There were {} failures to decomment import statements\n".format(failures))

        elif args[0].lower() == "--move-foo":
            src_root = os.path.join(get_repo_root(), "src", "command_modules")

            py_count = 0
            yaml_count = 0
            failures = 0

            for root, dirs, files in os.walk(src_root):
                for file in files:
                    if file.endswith("foo.py") and os.path.dirname(root).endswith("command_modules") and os.path.join("build", "lib") not in root:
                        src = os.path.join(root, file)
                        dst = os.path.join(root, "_help.py")
                        print("Found {}\n".format(src))
                        print("Renaming {}\n\tto {}\n.".format(src, dst))
                        os.rename(src, dst)
                        py_count +=1

                        src = os.path.join(root, "__init__.py")
                        dst = os.path.join(root, "__init__2.py")

                        success = decomment_import_help(src, dst)

                        if success:
                            os.remove(src)
                            os.rename(dst, src)
                            print("De-commented out import in {}\n".format(src))
                        else:
                            os.remove(dst)
                            print("Failed to de-comment out import in {}\n".format(src))
                            failures+=1

            print("Renamed {} foo.py files to _help.py.\n".format(py_count))
            print("There were {} failures to decomment import statements\n".format(failures))

        elif args[0].lower() == "--add-extensions":
            command = "az extension list-available --query [].name -o tsv"
            completed = subprocess.run(command.split(), stdout=subprocess.PIPE, universal_newlines=True)
            if completed.returncode == 0:
                num_installed = 0
                extensions = completed.stdout.splitlines()
                for ext in extensions:
                    success = install_extension(ext)
                    if success:
                        num_installed += 1

                print("Installed {} extensions".format(num_installed))

        elif args[0].lower() == "--remove-extensions":
            command = "az extension list --query [].name -o tsv"
            completed = subprocess.run(command.split(), stdout=subprocess.PIPE, universal_newlines=True)
            if completed.returncode == 0:
                num_installed = 0
                extensions = completed.stdout.splitlines()
                for ext in extensions:
                    success = uninstall_extension(ext)
                    if success:
                        num_installed += 1

                print("Uninstalled {} extensions".format(num_installed))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest
from uuid import uuid4

from azure.cli.command_modules.resource._utils import split_resource_id


class TestSplitResourceId(unittest.TestCase):
    def test_subscription_level_id(self):
        subscription_id = uuid4()
        role_assignment_id = uuid4()

        scope, relative_resource_id = split_resource_id(
            f"/subscriptions/{subscription_id}/providers/Microsoft.Authorization/roleAssignment/{role_assignment_id}"
        )

        self.assertEqual(scope, f"/subscriptions/{subscription_id}")
        self.assertEqual(relative_resource_id, f"Microsoft.Authorization/roleAssignment/{role_assignment_id}")

    def test_resource_group_level_id(self):
        subscription_id = uuid4()

        scope, relative_resource_id = split_resource_id(
            f"/subscriptions/{subscription_id}/resourceGroups/test-what-if-rg/providers/Microsoft.Sql/servers/dbserver"
        )

        self.assertEqual(scope, f"/subscriptions/{subscription_id}/resourceGroups/test-what-if-rg")
        self.assertEqual(relative_resource_id, "Microsoft.Sql/servers/dbserver")

    def test_resource_group_id(self):
        subscription_id = uuid4()

        scope, relative_resource_id = split_resource_id(
            f"/subscriptions/{subscription_id}/resourceGroups/test-what-if-rg"
        )

        self.assertEqual(scope, f"/subscriptions/{subscription_id}")
        self.assertEqual(relative_resource_id, "resourceGroups/test-what-if-rg")

    def test_management_group_level_id(self):
        role_assignment_id = uuid4()

        scope, relative_resource_id = split_resource_id(
            f"/providers/Microsoft.Management/ManagementGroups/myManagementGroup/providers/Microsoft.Authorization/roleAssignments/{role_assignment_id}"
        )

        self.assertEqual(scope, "/providers/Microsoft.Management/ManagementGroups/myManagementGroup")
        self.assertEqual(relative_resource_id, f"Microsoft.Authorization/roleAssignments/{role_assignment_id}")

    def test_management_group_id(self):
        scope, relative_resource_id = split_resource_id(
            "/providers/Microsoft.Management/ManagementGroups/myManagementGroup"
        )

        self.assertEqual(scope, "/")
        self.assertEqual(relative_resource_id, "Microsoft.Management/ManagementGroups/myManagementGroup")

    def test_tenant_level_id(self):
        role_assignment_id = uuid4()

        scope, relative_resource_id = split_resource_id(
            f"/providers/Microsoft.Authorization/roleAssignments/{role_assignment_id}"
        )

        self.assertEqual(scope, "/")
        self.assertEqual(relative_resource_id, f"Microsoft.Authorization/roleAssignments/{role_assignment_id}")



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from knack.util import CLIError
from msal import ClientApplication


class AdalRefreshTokenBasedClientApplication(ClientApplication):
    """
    This is added only for vmssh feature.
    It is a temporary solution and will deprecate after MSAL adopted completely.
    """
    def _acquire_token_silent_by_finding_rt_belongs_to_me_or_my_family(
            self, authority, scopes, account, **kwargs):
        # pylint: disable=line-too-long
        return self._acquire_token_silent_by_finding_specific_refresh_token(
            authority, scopes, None, **kwargs)

    def _acquire_token_silent_by_finding_specific_refresh_token(
            self, authority, scopes, query,
            rt_remover=None, break_condition=lambda response: False, **kwargs):
        refresh_token = kwargs.get('refresh_token', None)
        client = self._build_client(self.client_credential, authority)
        if 'refresh_token' in kwargs:
            kwargs.pop('refresh_token')
        if 'force_refresh' in kwargs:
            kwargs.pop('force_refresh')
        if 'correlation_id' in kwargs:
            kwargs.pop('correlation_id')
        response = client.obtain_token_by_refresh_token(refresh_token, scope=scopes, **kwargs)
        if "error" in response:
            raise CLIError("Get token failed. {error}: {error_description}".format(**response))
        return response



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


import os

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, record_only

TEST_DIR = os.path.abspath(os.path.join(os.path.abspath(__file__), '..'))


class SynapseScenarioTests(ScenarioTest):
    location = "northeurope"

    @ResourceGroupPreparer(name_prefix='synapse-cli', random_name_length=16)
    def test_workspaces(self, resource_group):
        # create a workspace
        self._create_workspace()

        # check workspace name
        self.cmd('az synapse workspace check-name --name {workspace}', checks=[
            self.check('available', False)
        ])

        # get workspace with workspace name
        workspace = self.cmd('az synapse workspace show --name {workspace} --resource-group {rg}', checks=[
            self.check('name', self.kwargs['workspace']),
            self.check('type', 'Microsoft.Synapse/workspaces'),
            self.check('provisioningState', 'Succeeded')
        ]).get_output_in_json()

        self.kwargs["workspace-id"] = workspace['id']

        # list all workspaces under a specific resource group
        self.cmd('az synapse workspace list --resource-group {rg}', checks=[
            self.check('[0].type', 'Microsoft.Synapse/workspaces')
        ])

        # update workspace
        self.cmd('az synapse workspace update --ids {workspace-id} --tags key1=value1', checks=[
            self.check('tags.key1', 'value1'),
            self.check('name', self.kwargs['workspace']),
            self.check('id', self.kwargs['workspace-id']),
            self.check('type', 'Microsoft.Synapse/workspaces'),
            self.check('provisioningState', 'Succeeded')
        ])

        # delete workspace with workspace name
        self.cmd('az synapse workspace delete --name {workspace} --resource-group {rg} --yes')
        import time
        time.sleep(120)
        self.cmd('az synapse workspace show --name {workspace} --resource-group {rg}', expect_failure=True)

    @ResourceGroupPreparer(name_prefix='synapse-cli', random_name_length=16)
    def test_managed_virtual_network_workspace(self):
        # test workspace with managed virtual network
        self._create_workspace("--enable-managed-virtual-network")
        self.cmd('az synapse workspace show --name {workspace} --resource-group {rg}', checks=[
            self.check('name', self.kwargs['workspace']),
            self.check('type', 'Microsoft.Synapse/workspaces'),
            self.check('provisioningState', 'Succeeded'),
            self.check('managedVirtualNetwork', 'default')
        ])

    @record_only()
    def test_spark_pool(self):
        self.kwargs.update({
            'location': 'eastus',
            'workspace': 'testsynapseworkspace',
            'rg': 'rg',
            'spark-pool': self.create_random_name(prefix='testpool', length=15),
            'spark-version': '2.4'
        })
        # create spark pool
        spark_pool = self.cmd('az synapse spark pool create --name {spark-pool} --spark-version {spark-version}'
                              ' --workspace {workspace} --resource-group {rg} --node-count 3 --node-size Medium',
                              checks=[
                                  self.check('name', self.kwargs['spark-pool']),
                                  self.check('type', 'Microsoft.Synapse/workspaces/bigDataPools'),
                                  self.check('provisioningState', 'Succeeded')
                              ]).get_output_in_json()

        self.kwargs['pool-id'] = spark_pool['id']

        # get spark pool with spark pool name
        self.cmd('az synapse spark pool show --name {spark-pool} --workspace {workspace} --resource-group {rg}',
                 checks=[
                     self.check('name', self.kwargs['spark-pool']),
                     self.check('type', 'Microsoft.Synapse/workspaces/bigDataPools'),
                     self.check('provisioningState', 'Succeeded')
                 ])

        # list all spark pools under the workspace
        self.cmd('az synapse spark pool list --workspace {workspace} --resource-group {rg}', checks=[
            self.check('[0].type', 'Microsoft.Synapse/workspaces/bigDataPools')
        ])

        # update spark pool
        self.cmd('az synapse spark pool update --ids {pool-id} --tags key1=value1', checks=[
            self.check('tags.key1', 'value1'),
            self.check('name', self.kwargs['spark-pool']),
            self.check('type', 'Microsoft.Synapse/workspaces/bigDataPools'),
            self.check('provisioningState', 'Succeeded')
        ])

        # delete spark pool with spark pool name
        self.cmd(
            'az synapse spark pool delete --name {spark-pool} --workspace {workspace} --resource-group {rg} --yes')
        self.cmd('az synapse spark pool show --name {spark-pool} --workspace {workspace} --resource-group {rg}',
                 expect_failure=True)

    @record_only()
    def test_sql_pool(self):
        self.kwargs.update({
            'location': 'eastus',
            'workspace': 'testsynapseworkspace',
            'rg': 'rg',
            'sql-pool': self.create_random_name(prefix='testsqlpool', length=15),
            'performance-level': 'DW1000c'
        })
        # create sql pool
        sql_pool = self.cmd(
            'az synapse sql pool create --name {sql-pool} --performance-level {performance-level} '
            '--workspace {workspace} --resource-group {rg}', checks=[
                self.check('name', self.kwargs['sql-pool']),
                self.check('type', 'Microsoft.Synapse/workspaces/sqlPools'),
                self.check('provisioningState', 'Succeeded'),
                self.check('status', 'Online')
            ]).get_output_in_json()

        self.kwargs['pool-id'] = sql_pool['id']

        # get sql pool with sql pool name
        self.cmd('az synapse sql pool show --name {sql-pool} --workspace {workspace} --resource-group {rg}',
                 checks=[
                     self.check('name', self.kwargs['sql-pool']),
                     self.check('type', 'Microsoft.Synapse/workspaces/sqlPools'),
                     self.check('provisioningState', 'Succeeded'),
                     self.check('status', 'Online')
                 ])

        # list all sql pools under the workspace
        self.cmd('az synapse sql pool list --workspace {workspace} --resource-group {rg}', checks=[
            self.check('[0].type', 'Microsoft.Synapse/workspaces/sqlPools')
        ])

        # update sql pool
        self.cmd('az synapse sql pool update --ids {pool-id} --tags key1=value1')

        # get sql pool with sql pool id
        self.cmd('az synapse sql pool show --ids {pool-id}',
                 checks=[
                     self.check('name', self.kwargs['sql-pool']),
                     self.check('type', 'Microsoft.Synapse/workspaces/sqlPools'),
                     self.check('tags.key1', 'value1'),
                     self.check('provisioningState', 'Succeeded'),
                     self.check('status', 'Online')
                 ])

        # pause sql pool
        self.cmd('az synapse sql pool pause --name {sql-pool} --workspace {workspace} --resource-group {rg}', checks=[])
        self.cmd('az synapse sql pool show --name {sql-pool} --workspace {workspace} --resource-group {rg}',
                 checks=[
                     self.check('name', self.kwargs['sql-pool']),
                     self.check('type', 'Microsoft.Synapse/workspaces/sqlPools'),
                     self.check('status', 'Paused')
                 ])

        # resume sql pool
        self.cmd('az synapse sql pool resume --name {sql-pool} --workspace {workspace} --resource-group {rg}',
                 checks=[])
        self.cmd('az synapse sql pool show --name {sql-pool} --workspace {workspace} --resource-group {rg}',
                 checks=[
                     self.check('name', self.kwargs['sql-pool']),
                     self.check('type', 'Microsoft.Synapse/workspaces/sqlPools'),
                     self.check('status', 'Online')
                 ])

        # delete sql pool with sql pool name
        self.cmd(
            'az synapse sql pool delete --name {sql-pool} --workspace {workspace} --resource-group {rg} --yes')

        self.cmd('az synapse sql pool show --name {sql-pool} --workspace {workspace} --resource-group {rg}',
                 expect_failure=True)

    @record_only()
    def test_sql_pool_restore_and_list_deleted(self):
        self.kwargs.update({
            'location': 'eastus',
            'workspace': 'testsynapseworkspace',
            'rg': 'rg',
            'sql-pool': 'sqlpoolcli1',
            'dest-sql-pool': self.create_random_name(prefix='destsqlpool', length=15),
            'restore-point-time': '2020-11-25T02:47:37'
        })

        # restore sql pool
        self.cmd('az synapse sql pool restore --name {sql-pool} --workspace-name {workspace} --resource-group {rg} '
                 '--dest-name {dest-sql-pool} --time {restore-point-time}',
                 checks=[
                     self.check('name', self.kwargs['dest-sql-pool'])
                 ])

        # get the new created sql pool
        self.cmd('az synapse sql pool show --name {dest-sql-pool} --workspace-name {workspace} --resource-group {rg}',
                 checks=[
                     self.check('name', self.kwargs['dest-sql-pool']),
                     self.check('type', 'Microsoft.Synapse/workspaces/sqlPools'),
                     self.check('provisioningState', 'Succeeded'),
                     self.check('status', 'Online')
                 ])

        # delete dest sql pool with dest sql pool name
        self.cmd(
            'az synapse sql pool delete --name {dest-sql-pool} --workspace-name {workspace} --resource-group {rg} --yes')

        # test list-deleted
        self.cmd('az synapse sql pool list-deleted --workspace-name {workspace} --resource-group {rg}',
                 checks=[
                     self.greater_than("length([])", 0)
                 ])

    @record_only()
    def test_sql_pool_classification_and_recommendation(self):
        self.kwargs.update({
            'location': 'eastus',
            'workspace': 'testsynapseworkspace',
            'rg': 'rg',
            'sql-pool': 'sqlpoolcli1',
            'schema': 'dbo',
            'table': 'Person',
            'column': 'phone',
            'label': 'Confidential',
            'information-type': '"Contact Info"'
        })

        # classification create
        self.cmd('az synapse sql pool classification create --name {sql-pool} --workspace-name {workspace} '
                 '--resource-group {rg} --schema {schema} --table {table} --column {column} '
                 '--label {label} --information-type {information-type}',
                 checks=[
                     self.check('labelName', self.kwargs['label'])
                 ])

        # classification show
        self.cmd('az synapse sql pool classification show --name {sql-pool} --workspace-name {workspace} '
                 '--resource-group {rg} --schema {schema} --table {table} --column {column}',
                 checks=[
                     self.check('labelName', self.kwargs['label'])
                 ])

        # classification list
        self.cmd('az synapse sql pool classification list --name {sql-pool} --workspace-name {workspace} '
                 '--resource-group {rg}',
                 checks=[
                     self.check('[0].labelName', self.kwargs['label'])
                 ])

        # classification update
        self.cmd('az synapse sql pool classification update --name {sql-pool} --workspace-name {workspace} '
                 '--resource-group {rg} --schema {schema} --table {table} --column {column} '
                 '--label {label} --information-type {information-type}')

        # classification delete
        self.cmd('az synapse sql pool classification delete --name {sql-pool} --workspace-name {workspace} '
                 '--resource-group {rg} --schema {schema} --table {table} --column {column}')

        # recommendation enable
        self.cmd('az synapse sql pool classification recommendation enable --name {sql-pool} '
                 '--workspace-name {workspace} --resource-group {rg} '
                 '--schema {schema} --table {table} --column {column}')

        # recommendation list
        self.cmd('az synapse sql pool classification recommendation list --name {sql-pool} '
                 '--workspace-name {workspace} --resource-group {rg}',
                 checks=[
                     self.greater_than("length([])", 0)
                 ])

        self.cmd('az synapse sql pool classification recommendation disable --name {sql-pool} '
                 '--workspace-name {workspace} --resource-group {rg} '
                 '--schema {schema} --table {table} --column {column}')

        # After disable the length of list should be equal with 0
        self.cmd('az synapse sql pool classification recommendation list --name {sql-pool} '
                 '--workspace-name {workspace} --resource-group {rg}',
                 checks=[
                     self.check("length([])", 0)
                 ])

    @record_only()
    def test_sql_pool_tde(self):
        self.kwargs.update({
            'location': 'northeurope',
            'workspace': 'testtde1',
            'rg': 'rg',
            'sql-pool': 'sqlpooltde',  # self.create_random_name(prefix='testsqlpool', length=15),
        })

        self.cmd(
            'az synapse sql pool tde set --status Enabled '
            '--name {sql-pool} --workspace-name {workspace} --resource-group {rg}')

        self.cmd('az synapse sql pool tde show --name {sql-pool} --workspace-name {workspace} --resource-group {rg}',
                 checks=[
                     self.check('name', "current"),
                     self.check('status', "Enabled")
                 ])

    @record_only()
    def test_sql_pool_threat_policy(self):
        self.kwargs.update({
            'location': 'eastus',
            'workspace': 'testsynapseworkspace',
            'rg': 'rg',
            'sql-pool': 'sqlpoolcli1',  # self.create_random_name(prefix='testsqlpool', length=15),
            'storage-account': 'zzystorageforsynapse'
        })

        self.cmd('az synapse sql pool threat-policy update --state Enabled --storage-account {storage-account} '
                 '--name {sql-pool} --workspace-name {workspace} --resource-group {rg}')

        self.cmd('az synapse sql pool threat-policy show '
                 '--name {sql-pool} --workspace-name {workspace} --resource-group {rg}',
                 checks=[
                     self.check('state', 'Enabled')
                 ])

    @record_only()
    def test_sql_pool_audit_policy(self):
        self.kwargs.update({
            'location': 'eastus',
            'workspace': 'testsynapseworkspace',
            'rg': 'rg',
            'sql-pool': 'sqlpoolcli1',
            'storage-account': 'teststorageforsynapse'
        })

        # test show command
        self.cmd('az synapse sql pool audit-policy show '
                 '--name {sql-pool} --workspace-name {workspace} --resource-group {rg}',
                 checks=[
                     self.check('state', 'Disabled')
                 ])

        # test validator
        self.cmd('az synapse sql pool audit-policy update '
                 '--name {sql-pool} --workspace-name {workspace} --resource-group {rg}', expect_failure=True)

        # test for updating state from Disabled to Enabled with storage and retention days
        self.cmd('az synapse sql pool audit-policy update --state Enabled --storage-account {storage-account} '
                 '--retention-days 7 --name {sql-pool} --workspace-name {workspace} --resource-group {rg}')
        self.cmd('az synapse sql pool audit-policy show '
                 '--name {sql-pool} --workspace-name {workspace} --resource-group {rg}',
                 checks=[
                     self.check('state', 'Enabled')
                 ])

        # test for updating state from Enabled to Disabled
        self.cmd('az synapse sql pool audit-policy update --state Disabled '
                 '--name {sql-pool} --workspace-name {workspace} --resource-group {rg}')
        self.cmd('az synapse sql pool audit-policy show '
                 '--name {sql-pool} --workspace-name {workspace} --resource-group {rg}',
                 checks=[
                     self.check('state', 'Disabled')
                 ])

    @record_only()
    def test_sql_aad_admin(self):
        self.kwargs.update({
            'location': 'eastus',
            'workspace': 'testsynapseworkspace',
            'rg': 'rg',
            'user-name': 'fakeuser',
            'object-id': '00000000-0000-0000-0000-000000000000',
            'user-email': 'fakeuser@fakedomain.com'
        })
        # Test create cmdlet
        self.cmd('az synapse sql ad-admin create --workspace-name {workspace} --resource-group {rg} '
                 '--display-name {user-name} --object-id {object-id}',
                 checks=[
                     self.check('login', self.kwargs['user-name'])
                 ])

        # Test show cmdlet
        self.cmd('az synapse sql ad-admin show --workspace-name {workspace} --resource-group {rg}',
                 checks=[
                     self.check('login', self.kwargs['user-name']),
                     self.check('name', 'activeDirectory')
                 ])

        # Test update cmdlet
        self.cmd('az synapse sql ad-admin update --workspace-name {workspace} --resource-group {rg} '
                 '--display-name {user-email}',
                 checks=[
                     self.check('login', self.kwargs['user-email'])
                 ])
        # Test delete cmdlet
        self.cmd('az synapse sql ad-admin delete --workspace-name {workspace} --resource-group {rg} -y')
        self.cmd('az synapse sql ad-admin show --workspace-name {workspace} --resource-group {rg}', expect_failure=True)

    @record_only()
    def test_sql_audit_policy(self):
        self.kwargs.update({
            'location': 'eastus',
            'workspace': 'testsynapseworkspace',
            'rg': 'rg',
            'storage-account': 'teststorageforsynapse'
        })
        # test show command
        self.cmd('az synapse sql audit-policy show --workspace-name {workspace} --resource-group {rg}',
                 checks=[
                     self.check('state', 'Disabled')
                 ])

        # test validator of this command
        self.cmd('az synapse sql audit-policy update --workspace-name {workspace} --resource-group {rg}',
                 expect_failure=True)

        # test for updating state from Disabled to Enabled with storage and retention days
        self.cmd('az synapse sql audit-policy update --state Enabled --storage-account {storage-account} '
                 '--retention-days 7 --workspace-name {workspace} --resource-group {rg}')
        self.cmd('az synapse sql audit-policy show --workspace-name {workspace} --resource-group {rg}',
                 checks=[
                     self.check('state', 'Enabled')
                 ])

        # test for updating state from Enabled to Disabled
        self.cmd('az synapse sql audit-policy update --state Disabled '
                 '--workspace-name {workspace} --resource-group {rg}')
        self.cmd('az synapse sql audit-policy show --workspace-name {workspace} --resource-group {rg}',
                 checks=[
                     self.check('state', 'Disabled')
                 ])

    @record_only()
    @ResourceGroupPreparer(name_prefix='synapse-cli', random_name_length=16)
    def test_ip_firewall_rules(self, resource_group):
        self.kwargs.update({
            'workspace': 'testsynapseworkspace',
            'rg': 'rg',
            'ruleName': self.create_random_name(prefix='rule', length=8),
            'startIpAddress': "0.0.0.0",
            'endIpAddress': "255.255.255.255",
            'secondIpAddress': "192.0.0.1"
        })

        # create a firewall rule
        self.cmd(
            'az synapse workspace firewall-rule create --name {ruleName} --workspace-name {workspace} '
            '--resource-group {rg} --start-ip-address {startIpAddress} --end-ip-address {endIpAddress}',
            checks=[
                self.check('name', self.kwargs['ruleName']),
                self.check('type', 'Microsoft.Synapse/workspaces/firewallRules'),
                self.check('provisioningState', 'Succeeded')
            ])

        # get a firewall rule
        self.cmd(
            'az synapse workspace firewall-rule show --name {ruleName} --workspace-name {workspace} '
            '--resource-group {rg}',
            checks=[
                self.check('name', self.kwargs['ruleName']),
                self.check('type', 'Microsoft.Synapse/workspaces/firewallRules'),
                self.check('provisioningState', 'Succeeded')
            ])
        # update a firewall rule
        self.cmd(
            'az synapse workspace firewall-rule update --name {ruleName} --workspace-name {workspace} '
            '--resource-group {rg} --start-ip-address {secondIpAddress}',
            checks=[
                self.check('name', self.kwargs['ruleName']),
                self.check('startIpAddress', self.kwargs['secondIpAddress']),
                self.check('type', 'Microsoft.Synapse/workspaces/firewallRules'),
                self.check('provisioningState', 'Succeeded')
            ])

        # list all firewall rules under a specific workspace
        self.cmd('az synapse workspace firewall-rule list --workspace-name {workspace} --resource-group {rg}',
                 checks=[
                     self.check('[0].type', 'Microsoft.Synapse/workspaces/firewallRules')
                 ])

        # delete a firewall rule
        self.cmd(
            'az synapse workspace firewall-rule delete --name {ruleName} --workspace-name {workspace} '
            '--resource-group {rg} --yes')
        import time
        time.sleep(20)
        self.cmd('az synapse workspace firewall-rule show --name {ruleName} --workspace-name {workspace} '
                 '--resource-group {rg}', expect_failure=True)

    @record_only()
    @ResourceGroupPreparer(name_prefix='synapse-cli', random_name_length=16)
    def test_spark_job(self, resource_group):
        self.kwargs.update({
            'spark-pool': 'testsparkpool',
            'workspace': 'testsynapseworkspace',
            'job': 'WordCount_Java',
            'main-definition-file': 'abfss://testfilesystem@adlsgen2account.dfs.core.windows.net/samples/java/wordcount/wordcount.jar',
            'main-class-name': 'WordCount',
            'arguments': [
                'abfss://testfilesystem@adlsgen2account.dfs.core.windows.net/samples/java/wordcount/shakespeare.txt',
                'abfss://testfilesystem@adlsgen2account.dfs.core.windows.net/samples/java/wordcount/result/'],
            'executors': 2,
            'executor-size': 'Medium'
        })

        # create a spark batch job
        batch_job = self.cmd('az synapse spark job submit --name {job} --workspace-name {workspace} '
                             '--spark-pool-name {spark-pool} --main-definition-file {main-definition-file} '
                             '--main-class-name {main-class-name} --arguments {arguments} '
                             '--executors {executors} --executor-size {executor-size}',
                             checks=[self.check('name', self.kwargs['job']),
                                     self.check('jobType', 'SparkBatch'),
                                     self.check('state', 'not_started')
                                     ]).get_output_in_json()

        self.kwargs['batch-id'] = batch_job['id']

        # get a spark batch job with batch id
        self.cmd('az synapse spark job show --livy-id {batch-id} --workspace-name {workspace} '
                 '--spark-pool-name {spark-pool}', checks=[self.check('id', self.kwargs['batch-id'])])

        # list all spark batch jobs under a specific spark pool
        self.cmd('az synapse spark job list --workspace-name {workspace} '
                 '--spark-pool-name {spark-pool}',
                 checks=[
                     self.check('sessions[0].jobType', 'SparkBatch')
                 ])

        # cancel a spark batch job with batch id
        self.cmd('az synapse spark job cancel --livy-id {batch-id} --workspace-name {workspace} '
                 '--spark-pool-name {spark-pool} --yes')
        import time
        time.sleep(60)
        self.cmd('az synapse spark job show --livy-id {batch-id} --workspace-name {workspace} '
                 '--spark-pool-name {spark-pool}',
                 checks=[
                     self.check('result', 'Cancelled')
                 ])

    @record_only()
    @ResourceGroupPreparer(name_prefix='synapse-cli', random_name_length=16)
    def test_spark_session_and_statements(self, resource_group):
        self.kwargs.update({
            'spark-pool': 'testsparkpool',
            'workspace': 'testsynapseworkspace',
            'job': self.create_random_name(prefix='clisession', length=14),
            'executor-size': 'Small',
            'executors': 2,
            'code': "\"import time\ntime.sleep(10)\nprint('hello from cli')\"",
            'language': 'pyspark'
        })

        # create a spark session
        create_result = self.cmd('az synapse spark session create --name {job} --workspace-name {workspace} '
                                 '--spark-pool-name {spark-pool} --executor-size {executor-size} '
                                 '--executors {executors}',
                                 checks=[
                                     self.check('jobType', 'SparkSession'),
                                     self.check('name', self.kwargs['job']),
                                     self.check('state', 'not_started')
                                 ]).get_output_in_json()

        self.kwargs['session-id'] = create_result['id']

        # wait for creating spark session
        import time
        time.sleep(360)

        # get a spark session
        self.cmd('az synapse spark session show --livy-id {session-id} --workspace-name {workspace} '
                 '--spark-pool-name {spark-pool}',
                 checks=[
                     self.check('id', self.kwargs['session-id']),
                     self.check('state', 'idle')
                 ])

        # list all spark session jobs under a specific spark pook
        self.cmd('az synapse spark session list --workspace-name {workspace} '
                 '--spark-pool-name {spark-pool}',
                 checks=[
                     self.check('sessions[0].jobType', 'SparkSession')
                 ])

        # reset spark session's timeout time
        self.cmd('az synapse spark session reset-timeout --livy-id {session-id} --workspace-name {workspace} '
                 '--spark-pool-name {spark-pool}')

        # create a spark session statement job
        statement = self.cmd('az synapse spark statement invoke --session-id {session-id} '
                             '--workspace-name {workspace} --spark-pool-name {spark-pool} '
                             '--code {code} --language {language}',
                             checks=[
                                 self.check('state', 'waiting')
                             ]).get_output_in_json()
        self.kwargs['statement-id'] = statement['id']
        time.sleep(10)

        # get a spark session statement
        self.cmd('az synapse spark statement show --livy-id {statement-id} --session-id {session-id} '
                 '--workspace-name {workspace} --spark-pool-name {spark-pool}',
                 checks=[
                     self.check('state', 'running')
                 ])

        # list all spark session statements under a specific spark session
        self.cmd('az synapse spark statement list --session-id {session-id} '
                 '--workspace-name {workspace} --spark-pool-name {spark-pool}',
                 checks=[
                     self.check('statements[0].state', 'running')
                 ])

        # cancel a spark session statement
        self.cmd('az synapse spark statement cancel --livy-id {statement-id} --session-id {session-id} '
                 '--workspace-name {workspace} --spark-pool-name {spark-pool}  --yes',
                 checks=[
                     self.check('msg', 'canceled')
                 ])

        # delete/cancel a spark session
        self.cmd('az synapse spark session cancel --livy-id {session-id} --workspace-name {workspace} '
                 '--spark-pool-name {spark-pool} --yes')
        import time
        time.sleep(120)
        self.cmd('az synapse spark session show --livy-id {session-id} --workspace-name {workspace} '
                 '--spark-pool-name {spark-pool}',
                 checks=[
                     self.check('state', 'killed')
                 ])

    @record_only()
    def test_access_control(self):
        self.kwargs.update({
            'workspace': 'testsynapseworkspace',
            'role': 'Sql Admin',
            'userPrincipal': 'username@microsoft.com',
            'servicePrincipal': 'http://username-sp'})

        self.cmd(
            'az synapse role definition list --workspace-name {workspace} ',
            checks=[
                self.check('length([])', 3)
            ])

        # get role definition
        role_definition_get = self.cmd(
            'az synapse role definition show --workspace-name {workspace} --role "{role}" ',
            checks=[
                self.check('name', self.kwargs['role'])
            ]).get_output_in_json()
        self.kwargs['roleId'] = role_definition_get['id']

        # create role assignment
        role_assignment_create = self.cmd(
            'az synapse role assignment create --workspace-name {workspace} --role "{role}" --assignee  {userPrincipal} ',
            checks=[
                self.check('roleId', self.kwargs['roleId'])
            ]).get_output_in_json()
        self.kwargs['roleAssignmentId'] = role_assignment_create['id']
        self.kwargs['roleId'] = role_assignment_create['roleId']
        self.kwargs['principalId'] = role_assignment_create['principalId']

        # get role assignment
        self.cmd(
            'az synapse role assignment show --workspace-name {workspace} --id {roleAssignmentId} ',
            checks=[
                self.check('roleId', self.kwargs['roleId']),
                self.check('principalId', self.kwargs['principalId'])
            ])

        # list role assignment by role
        self.cmd(
            'az synapse role assignment list --workspace-name {workspace} --role "{role}" ',
            checks=[
                self.check('length([])', 2)
            ])

        # list role assignment by userPrincipal
        self.cmd(
            'az synapse role assignment list --workspace-name {workspace} --assignee {userPrincipal} ',
            checks=[
                self.check('length([])', 2)
            ])

        # list role assignment by servicePrincipal
        self.cmd(
            'az synapse role assignment list --workspace-name {workspace} --assignee {servicePrincipal} ',
            checks=[
                self.check('length([])', 1)
            ])

        # list role assignment by object_id
        self.cmd(
            'az synapse role assignment list --workspace-name {workspace} --assignee {principalId} ',
            checks=[
                self.check('length([])', 2)
            ])

        # delete role assignment
        self.cmd(
            'az synapse role assignment delete --workspace-name {workspace} --ids {roleAssignmentId} -y ')
        self.cmd(
            'az synapse role assignment show --workspace-name {workspace} --id {roleAssignmentId} ',
            expect_failure=True)

    def _create_workspace(self, *additional_create_params):
        self.kwargs.update({
            'workspace': self.create_random_name(prefix='clitest', length=16),
            'location': self.location,
            'file-system': 'testfilesystem',
            'login-user': 'cliuser1',
            'login-password': self.create_random_name(prefix='Pswd1', length=16)
        })

        # Create adlsgen2
        self._create_storage_account()

        # Wait some time to improve robustness
        if self.is_live or self.in_recording:
            import time
            time.sleep(60)

        # create synapse workspace
        self.cmd(
            'az synapse workspace create --name {workspace} --resource-group {rg} --storage-account {storage-account} '
            '--file-system {file-system} --sql-admin-login-user {login-user} '
            '--sql-admin-login-password {login-password}'
            ' --location {location} ' + ' '.join(additional_create_params), checks=[
                self.check('name', self.kwargs['workspace']),
                self.check('type', 'Microsoft.Synapse/workspaces'),
                self.check('provisioningState', 'Succeeded')
            ])

    def _create_storage_account(self):
        self.kwargs.update({
            'location': self.location,
            'storage-account': self.create_random_name(prefix='adlsgen2', length=16)
        })

        # Wait some time to improve robustness
        if self.is_live or self.in_recording:
            import time
            time.sleep(60)

        # create storage account
        self.cmd(
            'az storage account create --name {storage-account} --resource-group {rg} --enable-hierarchical-namespace true --location {location}',
            checks=[
                self.check('name', self.kwargs['storage-account']),
                self.check('type', 'Microsoft.Storage/storageAccounts'),
                self.check('provisioningState', 'Succeeded')
            ])

    @record_only()
    @ResourceGroupPreparer(name_prefix='synapse-cli', random_name_length=16)
    def test_linked_service(self):
        self.kwargs.update({
            'name': 'linkedservice'})

        # create a workspace
        self._create_workspace()
        # create firewall rule
        self.cmd(
            'az synapse workspace firewall-rule create --resource-group {rg} --name allowAll --workspace-name {workspace} '
            '--start-ip-address 0.0.0.0 --end-ip-address 255.255.255.255', checks=[
                self.check('provisioningState', 'Succeeded')
            ]
        )
        import time
        time.sleep(20)

        # create linked service
        self.cmd(
            'az synapse linked-service create --workspace-name {workspace} --name {name} --file @src/azure-cli/azure/cli/command_modules/synapse/tests/latest/assets/linkedservice.json',
            checks=[
                self.check('name', self.kwargs['name'])
            ])

        # get linked service
        self.cmd(
            'az synapse linked-service show --workspace-name {workspace} --name {name}',
            checks=[
                self.check('name', self.kwargs['name'])
            ])

        # list linked service
        self.cmd(
            'az synapse linked-service list --workspace-name {workspace}',
            checks=[
                self.check('[0].type', 'Microsoft.Synapse/workspaces/linkedservices')
            ])

        # delete linked service
        self.cmd(
            'az synapse linked-service delete --workspace-name {workspace} --name {name} -y')
        self.cmd(
            'az synapse linked-service show --workspace-name {workspace} --name {name}',
            expect_failure=True)

    @record_only()
    @ResourceGroupPreparer(name_prefix='synapse-cli', random_name_length=16)
    def test_dataset(self):
        self.kwargs.update({
            'name': 'dataset'})

        # create a workspace
        self._create_workspace()
        # create firewall rule
        self.cmd(
            'az synapse workspace firewall-rule create --resource-group {rg} --name allowAll --workspace-name {workspace} '
            '--start-ip-address 0.0.0.0 --end-ip-address 255.255.255.255', checks=[
                self.check('provisioningState', 'Succeeded')
            ]
        )
        import time
        time.sleep(20)

        self.kwargs['file'] = ('{\\"properties\\":{\\"linkedServiceName\\":{\\"referenceName\\":\\"' + self.kwargs[
            'workspace'] + '-WorkspaceDefaultStorage\\",'
                           '\\"type\\":\\"LinkedServiceReference\\"},\\"type\\":\\"Orc\\",\\"typeProperties\\":{\\"location\\":{\\"type\\":\\"AzureBlobFSLocation\\"}}}}')

        # create dataset
        self.cmd(
            'az synapse dataset create --workspace-name {workspace} --name {name} --file {file}',
            checks=[
                self.check('name', self.kwargs['name'])
            ])

        # get dataset
        self.cmd(
            'az synapse dataset show --workspace-name {workspace} --name {name}',
            checks=[
                self.check('name', self.kwargs['name'])
            ])

        # list dataset
        self.cmd(
            'az synapse dataset list --workspace-name {workspace}',
            checks=[
                self.check('[0].type', 'Microsoft.Synapse/workspaces/datasets')
            ])

        # delete dataset
        self.cmd(
            'az synapse dataset delete --workspace-name {workspace} --name {name} -y')
        self.cmd(
            'az synapse dataset show --workspace-name {workspace} --name {name}',
            expect_failure=True)

    @record_only()
    def test_pipeline(self):
        self.kwargs.update({
            'workspace': 'testsynapseworkspace',
            'name': 'pipeline'})

        # create pipeline
        self.cmd(
            'az synapse pipeline create --workspace-name {workspace} --name {name} --file @src/azure-cli/azure/cli/command_modules/synapse/tests/latest/assets/pipeline.json',
            checks=[
                self.check('name', self.kwargs['name'])
            ])

        # get pipeline
        self.cmd(
            'az synapse pipeline show --workspace-name {workspace} --name {name}',
            checks=[
                self.check('name', self.kwargs['name'])
            ])

        # list pipeline
        self.cmd(
            'az synapse pipeline list --workspace-name {workspace}',
            checks=[
                self.check('[0].type', 'Microsoft.Synapse/workspaces/pipelines')
            ])

        # create pipeline run
        pipeline_run = self.cmd(
            'az synapse pipeline create-run --workspace-name {workspace} --name {name}').get_output_in_json()
        self.kwargs['runId'] = pipeline_run['runId']

        # cancel pipeline run
        self.cmd(
            'az synapse pipeline-run cancel --workspace-name {workspace} --run-id {runId} -y')
        import time
        time.sleep(20)

        # get pipeline run by run id
        self.cmd(
            'az synapse pipeline-run show --workspace-name {workspace} --run-id {runId}',
            checks=[
                self.check('status', 'Cancelled')
            ])

        # get pipeline run by workspace
        self.cmd(
            'az synapse pipeline-run query-by-workspace --workspace-name {workspace} '
            '--last-updated-after 2020-09-01T00:36:44.3345758Z --last-updated-before 2020-10-16T00:36:44.3345758Z')

        # get acticity run
        self.cmd(
            'az synapse activity-run query-by-pipeline-run --workspace-name {workspace} --name {name} --run-id {runId} '
            '--last-updated-after 2020-09-01T00:36:44.3345758Z --last-updated-before 2020-10-16T00:36:44.3345758Z')

        # delete pipeline
        self.cmd(
            'az synapse pipeline delete --workspace-name {workspace} --name {name} -y')
        self.cmd(
            'az synapse pipeline show --workspace-name {workspace} --name {name}',
            expect_failure=True)

    @record_only()
    def test_trigger(self):
        self.kwargs.update({
            'workspace': 'testsynapseworkspace',
            'name': 'trigger',
            'event-trigger': 'EventTrigger',
            'tumbling-window-trigger': 'TumblingWindowTrigger',
            'run-id': '08586024051698130326966471413CU40'})

        # create trigger
        self.cmd(
            'az synapse trigger create --workspace-name {workspace} --name {name} --file @src/azure-cli/azure/cli/command_modules/synapse/tests/latest/assets/trigger.json',
            checks=[
                self.check('name', self.kwargs['name'])
            ])

        # get trigger
        self.cmd(
            'az synapse trigger show --workspace-name {workspace} --name {name}',
            checks=[
                self.check('name', self.kwargs['name'])
            ])

        # list trigger
        self.cmd(
            'az synapse trigger list --workspace-name {workspace}',
            checks=[
                self.check('[0].type', 'Microsoft.Synapse/workspaces/triggers')
            ])

        # delete trigger
        self.cmd(
            'az synapse trigger delete --workspace-name {workspace} --name {name} -y')
        self.cmd(
            'az synapse trigger show --workspace-name {workspace} --name {name}',
            expect_failure=True)

        # subscribe to event
        self.cmd(
            'az synapse trigger subscribe-to-event --workspace-name {workspace} --name {event-trigger}',
            checks=[
                self.check('status', 'Provisioning')
            ])
        import time
        time.sleep(20)

        # get event subscription status
        self.cmd(
            'az synapse trigger get-event-subscription-status --workspace-name {workspace} --name {event-trigger}',
            checks=[
                self.check('status', 'Enabled')
            ])

        # unsubscribe from event
        self.cmd(
            'az synapse trigger unsubscribe-from-event --workspace-name {workspace} --name {event-trigger}',
            checks=[
                self.check('status', 'Deprovisioning')
            ])

        # start a trigger
        self.cmd(
            'az synapse trigger start --workspace-name {workspace} --name {tumbling-window-trigger}')

        # get trigger run by workspace
        self.cmd(
            'az synapse trigger-run query-by-workspace --workspace-name {workspace} '
            '--last-updated-after 2020-09-01T00:36:44.3345758Z --last-updated-before 2020-10-01T00:36:44.3345758Z')

        # rerun a trigger
        self.cmd(
            'az synapse trigger-run rerun --workspace-name {workspace} --name {tumbling-window-trigger} --run-id {run-id}')

        # stop a trigger
        self.cmd(
            'az synapse trigger stop --workspace-name {workspace} --name {tumbling-window-trigger}')

    @record_only()
    def test_data_flow(self):
        self.kwargs.update({
            'workspace': 'testsynapseworkspace',
            'name': 'dataflow'})

        # create data flow
        self.cmd(
            'az synapse data-flow create --workspace-name {workspace} --name {name} --file @src/azure-cli/azure/cli/command_modules/synapse/tests/latest/assets/dataflow.json',
            checks=[
                self.check('name', self.kwargs['name'])
            ])

        # get data flow
        self.cmd(
            'az synapse data-flow show --workspace-name {workspace} --name {name}',
            checks=[
                self.check('name', self.kwargs['name'])
            ])

        # list data flow
        self.cmd(
            'az synapse data-flow list --workspace-name {workspace}',
            checks=[
                self.check('[0].type', 'Microsoft.Synapse/workspaces/dataflows')
            ])

        # delete data flow
        self.cmd(
            'az synapse data-flow delete --workspace-name {workspace} --name {name} -y')
        self.cmd(
            'az synapse data-flow show --workspace-name {workspace} --name {name}',
            expect_failure=True)

    @record_only()
    def test_notebook(self):
        self.kwargs.update({
            'workspace': 'testsynapseworkspace',
            'name': 'notebook',
            'spark-pool': 'testpool'})

        # create notebook
        self.cmd(
            'az synapse notebook create --workspace-name {workspace} --name {name} --file @src/azure-cli/azure/cli/command_modules/synapse/tests/latest/assets/notebook.ipynb '
            '--spark-pool-name {spark-pool}',
            checks=[
                self.check('name', self.kwargs['name'])
            ])

        # get notebook
        self.cmd(
            'az synapse notebook show --workspace-name {workspace} --name {name}',
            checks=[
                self.check('name', self.kwargs['name'])
            ])

        # list notebook
        self.cmd(
            'az synapse notebook list --workspace-name {workspace}',
            checks=[
                self.check('[0].type', 'Microsoft.Synapse/workspaces/notebooks')
            ])

        # export notebook
        self.kwargs['output-folder'] = os.getcwd()
        self.cmd(
            'az synapse notebook export --workspace-name {workspace} --name {name} '
            '--output-folder "{output-folder}"')
        file_path = os.path.join(self.kwargs['output-folder'], self.kwargs['name'] + '.ipynb')
        self.assertTrue(os.path.isfile(file_path))
        os.remove(file_path)

        # delete notebook
        self.cmd(
            'az synapse notebook delete --workspace-name {workspace} --name {name} -y')
        self.cmd(
            'az synapse notebook show --workspace-name {workspace} --name {name}',
            expect_failure=True)

    @record_only()
    def test_integration_runtime(self):
        self.kwargs.update({
            'rg': 'rg',
            'workspace': 'testsynapseworkspace',
            'name': 'integrationruntime',
            'type': 'Managed',
            'selfhosted-integration-runtime': 'SelfHostedIntegrationRuntime',
            'node': 'testnode'})

        # create integration runtime
        self.cmd(
            'az synapse integration-runtime create --resource-group {rg} --workspace-name {workspace} --name {name} --type {type}',
            checks=[
                self.check('name', self.kwargs['name'])
            ])

        # get integration runtime
        self.cmd(
            'az synapse integration-runtime show --resource-group {rg} --workspace-name {workspace} --name {name}',
            checks=[
                self.check('name', self.kwargs['name'])
            ])

        # list integration runtime
        self.cmd(
            'az synapse integration-runtime list --resource-group {rg} --workspace-name {workspace}',
            checks=[
                self.check('[0].type', 'Microsoft.Synapse/workspaces/integrationruntimes')
            ])

        # delete integration runtime
        self.cmd(
            'az synapse integration-runtime delete --resource-group {rg} --workspace-name {workspace} --name {name} -y')
        self.cmd(
            'az synapse integration-runtime show --resource-group {rg} --workspace-name {workspace} --name {name}',
            expect_failure=True)

        # upgrade self-hosted integration runtime
        self.cmd(
            'az synapse integration-runtime upgrade --resource-group {rg} --workspace-name {workspace} --name {selfhosted-integration-runtime}')

        # get keys for a self-hosted integration runtime
        key = self.cmd(
            'az synapse integration-runtime list-auth-key --resource-group {rg} --workspace-name {workspace} --name {selfhosted-integration-runtime}').get_output_in_json()
        assert key['authKey1'] is not None
        assert key['authKey2'] is not None

        # regenerate self-hosted integration runtime key
        key = self.cmd(
            'az synapse integration-runtime regenerate-auth-key --resource-group {rg} --workspace-name {workspace} --name {selfhosted-integration-runtime} '
            '--key-name authKey1').get_output_in_json()
        assert key['authKey1'] is not None
        assert key['authKey2'] is None

        # get metric data for a self-hosted integration runtime
        self.cmd(
            'az synapse integration-runtime get-monitoring-data --resource-group {rg} --workspace-name {workspace} --name {selfhosted-integration-runtime}',
            checks=[
                self.check('name', self.kwargs['selfhosted-integration-runtime'])
            ])

        # get self-hosted integration runtime node information
        self.cmd(
            'az synapse integration-runtime-node show --resource-group {rg} --workspace-name {workspace} --name {selfhosted-integration-runtime} '
            '--node-name {node}',
            checks=[
                self.check('nodeName', self.kwargs['node'])
            ])

        # update self-hosted integration runtime node
        self.cmd(
            'az synapse integration-runtime-node update --resource-group {rg} --workspace-name {workspace} --name {selfhosted-integration-runtime} '
            '--node-name {node}',
            checks=[
                self.check('nodeName', self.kwargs['node'])
            ])

        # get self-hosted integration runtime node ip
        self.cmd(
            'az synapse integration-runtime-node get-ip-address --resource-group {rg} --workspace-name {workspace} --name {selfhosted-integration-runtime} '
            '--node-name {node}')

        # sync credentials among integration runtime nodes
        self.cmd(
            'az synapse integration-runtime sync-credentials --resource-group {rg} --workspace-name {workspace} --name {selfhosted-integration-runtime}')

        # get connection info
        self.cmd(
            'az synapse integration-runtime get-connection-info --resource-group {rg} --workspace-name {workspace} --name {selfhosted-integration-runtime}')

        # get status
        self.cmd(
            'az synapse integration-runtime get-status --resource-group {rg} --workspace-name {workspace} --name {selfhosted-integration-runtime}',
            checks=[
                self.check('name', self.kwargs['selfhosted-integration-runtime'])
            ])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.decorators import Completer
from azure.cli.core.extension import get_extension_names

from azure.cli.core.extension.operations import get_index_extensions


@Completer
def extension_name_completion_list(cmd, prefix, namespace, **kwargs):  # pylint: disable=unused-argument
    return get_extension_names()


@Completer
def extension_name_from_index_completion_list(cmd, prefix, namespace, **kwargs):  # pylint: disable=unused-argument
    return get_index_extensions().keys()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import unittest

from azure.cli.testsdk import record_only
from azure.cli.testsdk import LiveScenarioTest, ResourceGroupPreparer

TEST_DIR = os.path.dirname(os.path.realpath(__file__))
TEMPLATE = os.path.join(TEST_DIR, 'lab_template.json').replace('\\', '\\\\')
ENV_PARAMETERS = os.path.join('@' + TEST_DIR, 'lab_template.json').replace('\\', '\\\\')
LAB_NAME = 'cliautomationlab'


class LabGalleryVMMgmtScenarioTest(LiveScenarioTest):

    @unittest.skip("The test is not working. Pending update.")
    @ResourceGroupPreparer(name_prefix='cliautomation')
    def test_lab_gallery_vm_mgmt(self, resource_group):
        self.kwargs.update({
            'linux_vm_name': 'ubuntuvm5367',
            'linux_image': '\"Ubuntu Server 16.04 LTS\"',
            'windows_vm_name': 'winvm5367',
            'windows_image': '\"Windows Server 2008 R2 SP1\"',
            'image_type': 'gallery',
            'size': 'Standard_DS1_v2',
            'password': 'SecretPassword123',
            'template': TEMPLATE,
            'lab_name': LAB_NAME
        })

        self.cmd('group deployment create -g {rg} --template-file {template}', checks=[
            self.check('properties.provisioningState', 'Succeeded')
        ])

        # Create claimable linux vm in the lab
        self.cmd('lab vm create -g {rg} --lab-name {lab_name} --name {linux_vm_name} --image {linux_image}'
                 ' --image-type {image_type} --size {size} --admin-password {password} --allow-claim', checks=[
                     self.is_empty()])

        self.cmd('lab vm show -g {rg} --lab-name {lab_name} --name {linux_vm_name}', checks=[
            self.check('name', '{linux_vm_name}'),
            self.check('provisioningState', 'Succeeded'),
            self.check('osType', 'Linux'),
            self.check('virtualMachineCreationSource', 'FromGalleryImage'),
            self.check('size', '{size}'),
            self.check('disallowPublicIpAddress', True),
            self.check('artifactDeploymentStatus.totalArtifacts', 0),
            self.check('galleryImageReference.publisher', 'Canonical')
        ])

        # Create windows vm in the lab
        self.cmd('lab vm create -g {rg} --lab-name {lab_name} --name {windows_vm_name} '
                 '--image {windows_image} --image-type {image_type} --size {size} --admin-password {password} '
                 '--allow-claim', checks=[
                     self.is_empty()])

        self.cmd('lab vm show -g {rg} --lab-name {lab_name} --name {windows_vm_name} ', checks=[
            self.check('name', '{windows_vm_name}'),
            self.check('provisioningState', 'Succeeded'),
            self.check('osType', 'Windows'),
            self.check('virtualMachineCreationSource', 'FromGalleryImage'),
            self.check('size', '{size}'),
            self.check('disallowPublicIpAddress', True),
            self.check('artifactDeploymentStatus.totalArtifacts', 0),
            self.check('galleryImageReference.publisher', 'MicrosoftWindowsServer')
        ])

        # List claimable vms
        self.cmd('lab vm list -g {rg} --lab-name {lab_name} --claimable', checks=[
            self.check('length(@)', 2)
        ])

        # claim a specific vm
        self.cmd('lab vm claim -g {rg} --lab-name {lab_name} --name {linux_vm_name}', checks=[
            self.is_empty()
        ])

        # List my vms - we have already claimed one VM
        self.cmd('lab vm list -g {rg} --lab-name {lab_name}', checks=[
            self.check('length(@)', 1)
        ])

        # claim any vm
        self.cmd('lab vm claim -g {rg} --lab-name {lab_name}', checks=[
            self.is_empty()
        ])

        # List my vms - we have claimed both VMs
        self.cmd('lab vm list -g {rg} --lab-name {lab_name}', checks=[
            self.check('length(@)', 2)
        ])

        # Delete all the vms
        self.cmd('lab vm delete -g {rg} --lab-name {lab_name} --name {linux_vm_name}', checks=[
            self.is_empty()
        ])
        self.cmd('lab vm delete -g {rg} --lab-name {lab_name} --name {windows_vm_name}', checks=[
            self.is_empty()
        ])

        # Delete the lab
        self.cmd('lab delete -g {rg} --name {lab_name}', checks=[
            self.is_empty()
        ])


class LabEnvironmentMgmtScenarioTest(LiveScenarioTest):

    @unittest.skip("The test is not working. Pending update.")
    @ResourceGroupPreparer(name_prefix='cliautomation01')
    def test_lab_environment_mgmt(self, resource_group):
        self.kwargs.update({
            'env_name': 'docdbenv',
            'arm_template': 'documentdb-webapp',
            'password': 'SecretPassword123',
            'template': TEMPLATE,
            'lab_name': LAB_NAME,
            'env_params': ENV_PARAMETERS
        })

        self.cmd('group deployment create -g {rg} --template-file {template}', checks=[
            self.check('properties.provisioningState', 'Succeeded')
        ])

        artifact_sources = self.cmd('lab artifact-source list -g {rg} --lab-name {lab_name}') \
            .get_output_in_json()

        self.kwargs.update({
            'artifact_source_name': artifact_sources[0]['name']
        })

        # Create environment in the lab
        self.cmd('lab environment create -g {rg} --lab-name {lab_name} --name {env_name} --arm-template '
                 '{arm_template} --artifact-source-name {artifact_source_name} --parameters {env_params}', checks=[
                     self.check('provisioningState', 'Succeeded'),
                     self.check('type', 'Microsoft.DevTestLab/labs/users/environments')])

        # Delete environment from the lab
        self.cmd('lab environment delete -g {rg} --lab-name {lab_name} --name {env_name}', checks=[
            self.is_empty()
        ])

        # Delete the lab
        self.cmd('lab delete -g {rg} --name {lab_name}', checks=[
            self.is_empty()
        ])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import os
import unittest
from azure.cli.testsdk import ScenarioTest, JMESPathCheck, ResourceGroupPreparer, live_only


@live_only()
class ResourceDeleteTests(ScenarioTest):
    @ResourceGroupPreparer(name_prefix='cli_test_delete_dependent_resources', location='southcentralus')
    def test_delete_dependent_resources(self, resource_group):
        vm_name = self.create_random_name('cli-test-vm', 30)
        tag_name = self.create_random_name('cli-test-tag', 20)
        username = 'ubuntu'
        password = self.create_random_name('Password#1', 30)

        self.cmd('vm create -n {} -g {} --image UbuntuLTS --tag {} '
                 '--admin-username {} --admin-password {} --authentication-type {} --nsg-rule None'
                 .format(vm_name, resource_group, tag_name, username, password, 'password'))

        rsrc_list = self.cmd('resource list --tag {} --query [].id'.format(tag_name)).get_output_in_json()
        self.cmd('resource delete --ids {}'.format(' '.join(rsrc_list)))
        self.cmd('resource wait --ids {} --deleted --timeout 300'.format(''.join(rsrc_list)))


if __name__ == '__main__':
    unittest.main()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import CheckResourceNameResult
    from ._models_py3 import ErrorDefinition
    from ._models_py3 import ErrorResponse
    from ._models_py3 import Location
    from ._models_py3 import LocationListResult
    from ._models_py3 import Operation
    from ._models_py3 import OperationDisplay
    from ._models_py3 import OperationListResult
    from ._models_py3 import ResourceName
    from ._models_py3 import Subscription
    from ._models_py3 import SubscriptionListResult
    from ._models_py3 import SubscriptionPolicies
    from ._models_py3 import TenantIdDescription
    from ._models_py3 import TenantListResult
except (SyntaxError, ImportError):
    from ._models import CheckResourceNameResult  # type: ignore
    from ._models import ErrorDefinition  # type: ignore
    from ._models import ErrorResponse  # type: ignore
    from ._models import Location  # type: ignore
    from ._models import LocationListResult  # type: ignore
    from ._models import Operation  # type: ignore
    from ._models import OperationDisplay  # type: ignore
    from ._models import OperationListResult  # type: ignore
    from ._models import ResourceName  # type: ignore
    from ._models import Subscription  # type: ignore
    from ._models import SubscriptionListResult  # type: ignore
    from ._models import SubscriptionPolicies  # type: ignore
    from ._models import TenantIdDescription  # type: ignore
    from ._models import TenantListResult  # type: ignore

from ._subscription_client_enums import (
    ResourceNameStatus,
    SpendingLimit,
    SubscriptionState,
)

__all__ = [
    'CheckResourceNameResult',
    'ErrorDefinition',
    'ErrorResponse',
    'Location',
    'LocationListResult',
    'Operation',
    'OperationDisplay',
    'OperationListResult',
    'ResourceName',
    'Subscription',
    'SubscriptionListResult',
    'SubscriptionPolicies',
    'TenantIdDescription',
    'TenantListResult',
    'ResourceNameStatus',
    'SpendingLimit',
    'SubscriptionState',
]



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

import os
from azure.cli.testsdk import ScenarioTest, record_only
from .. import try_manual, raise_if, calc_coverage


TEST_DIR = os.path.abspath(os.path.join(os.path.abspath(__file__), '..'))


# Env setup
# @try_manual
# def setup(test):
#     pass


# EXAMPLE: /BillingAccounts/get/BillingAccounts
@try_manual
def step__billingaccounts_get_billingaccounts(test):
    test.cmd('az billing account show '
             '--name "{myBillingAccount}"',
             checks=[])


# EXAMPLE: /BillingAccounts/get/BillingAccountsList
@try_manual
def step__billingaccounts_get_billingaccountslist(test):
    test.cmd('az billing account list',
             checks=[])


# EXAMPLE: /BillingAccounts/get/BillingAccountsListWithExpand
@try_manual
def step__billingaccounts_get(test):
    test.cmd('az billing account list '
             '--expand "soldTo,billingProfiles,billingProfiles/invoiceSections"',
             checks=[])


# EXAMPLE: /BillingAccounts/get/BillingAccountsListWithExpandForEnrollmentDetails
@try_manual
def step__billingaccounts_get2(test):
    test.cmd('az billing account list '
             '--expand "enrollmentDetails,departments,enrollmentAccounts"',
             checks=[])


# EXAMPLE: /BillingAccounts/get/BillingAccountWithExpand
@try_manual
def step__billingaccounts_get_billingaccountwithexpand(test):
    test.cmd('az billing account show '
             '--expand "soldTo,billingProfiles,billingProfiles/invoiceSections" '
             '--name "{myBillingAccount}"',
             checks=[])


# EXAMPLE: /BillingAccounts/patch/UpdateBillingAccount
@try_manual
def step__billingaccounts_patch_updatebillingaccount(test):
    test.cmd('az billing account update '
             '--name "{myBillingAccount}" '
             '--display-name "Test Account" '
             '--sold-to address-line1="Test Address 1" city="Redmond" company-name="Contoso" country="US" '
             'first-name="Test" last-name="User" postal-code="12345" region="WA"',
             checks=[
                 test.check("name", "{myBillingAccount}", case_sensitive=False),
                 test.check("displayName", "Test Account", case_sensitive=False),
                 test.check("soldTo.addressLine1", "Test Address 1", case_sensitive=False),
                 test.check("soldTo.city", "Redmond", case_sensitive=False),
                 test.check("soldTo.companyName", "Contoso", case_sensitive=False),
                 test.check("soldTo.country", "US", case_sensitive=False),
                 test.check("soldTo.firstName", "Test", case_sensitive=False),
                 test.check("soldTo.lastName", "User", case_sensitive=False),
                 test.check("soldTo.postalCode", "12345", case_sensitive=False),
                 test.check("soldTo.region", "WA", case_sensitive=False),
             ])


# EXAMPLE: /Agreements/get/AgreementByName
@try_manual
def step__agreements_get_agreementbyname(test):
    test.cmd('az billing agreement show '
             '--name "{myAgreement}" '
             '--account-name "{myBillingAccount}"',
             checks=[])


# EXAMPLE: /Agreements/get/AgreementsListByBillingAccount
@try_manual
def step__agreements_get(test):
    test.cmd('az billing agreement list '
             '--account-name "{myBillingAccount}"',
             checks=[])


# EXAMPLE: /BillingProfiles/put/CreateBillingProfile
@try_manual
def step__billingprofiles_put_createbillingprofile(test):
    test.cmd('az billing profile create '
             '--account-name "{myBillingAccount}" '
             '--name "{myBillingProfile}" '
             '--bill-to address-line1="Test Address 1" city="Redmond" country="US" first-name="Test" last-name="User" '
             'postal-code="12345" region="WA" '
             '--display-name "Finance" '
             '--enabled-azure-plans sku-id="0001" '
             '--enabled-azure-plans sku-id="0002" '
             '--invoice-email-opt-in true '
             '--po-number "ABC12345"',
             checks=[])


# EXAMPLE: /BillingProfiles/get/BillingProfile
@try_manual
def step__billingprofiles_get_billingprofile(test):
    test.cmd('az billing profile show '
             '--account-name "{myBillingAccount}" '
             '--name "{myBillingProfile}"',
             checks=[
                 test.check("name", "{myBillingProfile}", case_sensitive=False),
                 test.check("invoiceEmailOptIn", True),
                 test.check("poNumber", "ABC12345", case_sensitive=False),
             ])


# EXAMPLE: /BillingProfiles/get/BillingProfilesListByBillingAccount
@try_manual
def step__billingprofiles_get(test):
    test.cmd('az billing profile list '
             '--account-name "{myBillingAccount}"',
             checks=[
                 test.check('length(@)', 1),
             ])


# EXAMPLE: /BillingProfiles/get/BillingProfilesListWithExpand
@try_manual
def step__billingprofiles_get2(test):
    test.cmd('az billing profile list '
             '--expand "invoiceSections" '
             '--account-name "{myBillingAccount}"',
             checks=[])


# EXAMPLE: /BillingProfiles/get/BillingProfileWithExpand
@try_manual
def step__billingprofiles_get_billingprofilewithexpand(test):
    test.cmd('az billing profile show '
             '--expand "invoiceSections" '
             '--account-name "{myBillingAccount}" '
             '--name "{myBillingProfile}"',
             checks=[
                 test.check("name", "{myBillingProfile}", case_sensitive=False),
                 test.check("invoiceEmailOptIn", True),
                 test.check("poNumber", "ABC12345", case_sensitive=False),
             ])


# EXAMPLE: /AvailableBalances/get/AvailableBalanceByBillingProfile
@try_manual
def step__availablebalances_get(test):
    test.cmd('az billing balance show '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}"',
             checks=[])


# EXAMPLE: /BillingRoleAssignments/get/InvoiceSectionRoleAssignmentList
@try_manual
def step__billingroleassignments_get(test):
    test.cmd('az billing role-assignment list '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}" '
             '--invoice-section-name "{myInvoiceSection}"',
             checks=[])


# EXAMPLE: /BillingPermissions/get/InvoiceSectionPermissionsList
@try_manual
def step__billingpermissions_get(test):
    test.cmd('az billing permission list '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}" '
             '--invoice-section-name "{myInvoiceSection}"',
             checks=[])


# EXAMPLE: /BillingRoleAssignments/get/BillingAccountRoleAssignmentList
@try_manual
def step__billingroleassignments_get2(test):
    test.cmd('az billing role-assignment list '
             '--account-name "{myBillingAccount}"',
             checks=[])


# EXAMPLE: /BillingRoleAssignments/get/BillingProfileRoleAssignmentList
@try_manual
def step__billingroleassignments_get3(test):
    test.cmd('az billing role-assignment list '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}"',
             checks=[])


# EXAMPLE: /BillingRoleAssignments/delete/BillingAccountRoleAssignmentDelete
@try_manual
def step__billingroleassignments_delete(test):
    test.cmd('az billing role-assignment delete -y '
             '--account-name "{myBillingAccount}" '
             '--name "{myBillingRoleAssignment}"',
             checks=[])


# EXAMPLE: /BillingRoleAssignments/delete/BillingProfileRoleAssignmentDelete
@try_manual
def step__billingroleassignments_delete2(test):
    test.cmd('az billing role-assignment delete -y '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}" '
             '--name "{myBillingRoleAssignment}"',
             checks=[])


# EXAMPLE: /BillingProperty/patch/UpdateBillingProperty
@try_manual
def step__billingproperty_patch_updatebillingproperty(test):
    test.cmd('az billing property update '
             '--cost-center "1010"',
             checks=[])


# EXAMPLE: /BillingRoleAssignments/delete/InvoiceSectionRoleAssignmentDelete
@try_manual
def step__billingroleassignments_delete3(test):
    test.cmd('az billing role-assignment delete -y '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}" '
             '--name "{myBillingRoleAssignment}" '
             '--invoice-section-name "{myInvoiceSection}"',
             checks=[])


# EXAMPLE: /BillingRoleDefinitions/get/BillingAccountRoleDefinitionsList
@try_manual
def step__billingroledefinitions_get(test):
    test.cmd('az billing role-definition list '
             '--account-name "{myBillingAccount}"',
             checks=[])


# EXAMPLE: /Customers/get/CustomersListByBillingAccount
@try_manual
def step__customers_get_customerslistbybillingaccount(test):
    test.cmd('az billing customer list '
             '--account-name "{myBillingAccount}"',
             checks=[])


# EXAMPLE: /Customers/get/CustomersListByBillingProfile
@try_manual
def step__customers_get_customerslistbybillingprofile(test):
    test.cmd('az billing customer list '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}"',
             checks=[])


# EXAMPLE: /Customers/get/CustomerWithExpand
@try_manual
def step__customers_get_customerwithexpand(test):
    test.cmd('az billing customer show '
             '--expand "enabledAzurePlans,resellers" '
             '--account-name "{myBillingAccount}" '
             '--name "{myCustomer}"',
             checks=[])


# EXAMPLE: /BillingPermissions/get/BillingProfilePermissionsList
@try_manual
def step__billingpermissions_get2(test):
    test.cmd('az billing permission list '
             '--account-name "{myBillingAccount}" '
             '--customer-name "{myCustomer}"',
             checks=[])


# EXAMPLE: /BillingRoleDefinitions/get/BillingProfileRoleDefinitionsList
@try_manual
def step__billingroledefinitions_get2(test):
    test.cmd('az billing role-definition list '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}"',
             checks=[])


# EXAMPLE: /BillingRoleDefinitions/get/InvoiceSectionRoleDefinitionsList
@try_manual
def step__billingroledefinitions_get3(test):
    test.cmd('az billing role-definition list '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}" '
             '--invoice-section-name "{myInvoiceSection}"',
             checks=[])


# EXAMPLE: /BillingSubscriptions/get/BillingSubscription
@try_manual
def step__billingsubscriptions_get_billingsubscription(test):
    test.cmd('az billing subscription show '
             '--account-name "{myBillingAccount}"',
             checks=[])


# EXAMPLE: /Customers/get/Customer
@try_manual
def step__customers_get_customer(test):
    test.cmd('az billing customer show '
             '--account-name "{myBillingAccount}" '
             '--name "{myCustomer}"',
             checks=[])


# EXAMPLE: /BillingSubscriptions/get/BillingSubscriptionsListByBillingAccount
@try_manual
def step__billingsubscriptions_get(test):
    test.cmd('az billing subscription list '
             '--account-name "{myBillingAccount}"',
             checks=[])


# EXAMPLE: /BillingSubscriptions/get/BillingSubscriptionsListByBillingProfile
@try_manual
def step__billingsubscriptions_get2(test):
    test.cmd('az billing subscription list '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}"',
             checks=[])


# EXAMPLE: /BillingSubscriptions/get/BillingSubscriptionsListByCustomer
@try_manual
def step__billingsubscriptions_get3(test):
    test.cmd('az billing subscription list '
             '--account-name "{myBillingAccount}" '
             '--customer-name "{myCustomer}"',
             checks=[])


# EXAMPLE: /BillingSubscriptions/get/BillingSubscriptionsListByInvoiceSection
@try_manual
def step__billingsubscriptions_get4(test):
    test.cmd('az billing subscription list '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}" '
             '--invoice-section-name "{myInvoiceSection}"',
             checks=[])


# EXAMPLE: /BillingProperty/get/BillingProperty
@try_manual
def step__billingproperty_get_billingproperty(test):
    test.cmd('az billing property show',
             checks=[])


# EXAMPLE: /BillingSubscriptions/patch/UpdateBillingProperty
@try_manual
def step__billingsubscriptions_patch(test):
    test.cmd('az billing subscription update '
             '--account-name "{myBillingAccount}" '
             '--cost-center "ABC1234"',
             checks=[])


# EXAMPLE: /BillingSubscriptions/post/MoveBillingSubscription
@try_manual
def step__billingsubscriptions_post2(test):
    test.cmd('az billing subscription move '
             '--account-name "{myBillingAccount}" '
             '--destination-invoice-section-id "/providers/Microsoft.Billing/billingAccounts/{billingAccountName}/billi'
             'ngProfiles/{billingProfileName}/invoiceSections/{newInvoiceSectionName}"',
             checks=[])


# EXAMPLE: /BillingSubscriptions/post/SubscriptionMoveValidateFailure
@try_manual
def step__billingsubscriptions_post(test):
    test.cmd('az billing subscription validate-move '
             '--account-name "{myBillingAccount}" '
             '--destination-invoice-section-id "/providers/Microsoft.Billing/billingAccounts/{billingAccountName}/billi'
             'ngProfiles/{billingProfileName}/invoiceSections/{newInvoiceSectionName}"',
             checks=[])


# EXAMPLE: /BillingSubscriptions/post/SubscriptionMoveValidateSuccess
@try_manual
def step__billingsubscriptions_post3(test):
    test.cmd('az billing subscription validate-move '
             '--account-name "{myBillingAccount}" '
             '--destination-invoice-section-id "/providers/Microsoft.Billing/billingAccounts/{billingAccountName}/billi'
             'ngProfiles/{billingProfileName}/invoiceSections/{newInvoiceSectionName}"',
             checks=[])


# EXAMPLE: /Instructions/get/Instruction
@try_manual
def step__instructions_get_instruction(test):
    test.cmd('az billing instruction show '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}" '
             '--name "{myInstruction}"',
             checks=[])


# EXAMPLE: /Invoices/get/BillingAccountInvoicesListWithRebillDetails
@try_manual
def step__invoices_get(test):
    test.cmd('az billing invoice list '
             '--account-name "{myBillingAccount}" '
             '--period-end-date "2018-06-30" '
             '--period-start-date "2018-01-01"',
             checks=[])


# EXAMPLE: /Invoices/get/BillingSubscriptionsListByBillingAccount
@try_manual
def step__invoices_get2(test):
    test.cmd('az billing invoice list '
             '--period-end-date "2018-06-30" '
             '--period-start-date "2018-01-01"',
             checks=[])


# EXAMPLE: /Invoices/get/CreditNote
@try_manual
def step__invoices_get_creditnote(test):
    test.cmd('az billing invoice show '
             '--account-name "{myBillingAccount}" '
             '--name "{myInvoice}"',
             checks=[])


# EXAMPLE: /BillingPermissions/get/BillingProfilePermissionsList
@try_manual
def step__billingpermissions_get2(test):
    test.cmd('az billing permission list '
             '--account-name "{myBillingAccount}" '
             '--customer-name "{myCustomer}"',
             checks=[])


# EXAMPLE: /Invoices/get/Invoice
@try_manual
def step__invoices_get_invoice(test):
    test.cmd('az billing invoice show '
             '--account-name "{myBillingAccount}" '
             '--name "{myInvoice}"',
             checks=[])


# EXAMPLE: /Invoices/get/InvoicesListByBillingProfile
@try_manual
def step__invoices_get_invoiceslistbybillingprofile(test):
    test.cmd('az billing invoice list '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}" '
             '--period-end-date "2018-06-30" '
             '--period-start-date "2018-01-01"',
             checks=[])


# EXAMPLE: /Instructions/get/InstructionsListByBillingProfile
@try_manual
def step__instructions_get(test):
    test.cmd('az billing instruction list '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}"',
             checks=[])


# EXAMPLE: /Instructions/put/PutInstruction
@try_manual
def step__instructions_put_putinstruction(test):
    test.cmd('az billing instruction create '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}" '
             '--name "{myInstruction}" '
             '--amount 5000 '
             '--end-date "2020-12-30T21:26:47.997Z" '
             '--start-date "2019-12-30T21:26:47.997Z"',
             checks=[
                 test.check("name", "{myInstruction}", case_sensitive=False),
                 test.check("amount", 5000),
                 test.check("endDate", "2020-12-30T21:26:47.997Z", case_sensitive=False),
                 test.check("startDate", "2019-12-30T21:26:47.997Z", case_sensitive=False),
             ])


# EXAMPLE: /Invoices/get/InvoicesListByBillingProfileWithRebillDetails
@try_manual
def step__invoices_get3(test):
    test.cmd('az billing invoice list '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}" '
             '--period-end-date "2018-06-30" '
             '--period-start-date "2018-01-01"',
             checks=[])


# EXAMPLE: /Invoices/get/BillingAccountInvoicesList
@try_manual
def step__invoices_get_billingaccountinvoiceslist(test):
    test.cmd('az billing invoice list '
             '--account-name "{myBillingAccount}" '
             '--period-end-date "2018-06-30" '
             '--period-start-date "2018-01-01"',
             checks=[])


# EXAMPLE: /Invoices/get/InvoiceWithRebillDetails
@try_manual
def step__invoices_get_invoicewithrebilldetails(test):
    test.cmd('az billing invoice show '
             '--account-name "{myBillingAccount}" '
             '--name "{myInvoice}"',
             checks=[])


# EXAMPLE: /Invoices/get/VoidInvoice
@try_manual
def step__invoices_get_voidinvoice(test):
    test.cmd('az billing invoice show '
             '--account-name "{myBillingAccount}" '
             '--name "{myInvoice}"',
             checks=[])


# EXAMPLE: /InvoiceSections/put/PutInvoiceSection
@try_manual
def step__invoicesections_put_putinvoicesection(test):
    test.cmd('az billing invoice section create '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}" '
             '--name "{myInvoiceSection}" '
             '--display-name "invoiceSection1" '
             '--labels costCategory="Support" pcCode="A123456"',
             checks=[
                 test.check("name", "{myInvoiceSection}", case_sensitive=False),
                 test.check("displayName", "invoiceSection1", case_sensitive=False),
                 test.check("labels.costCategory", "Support", case_sensitive=False),
                 test.check("labels.pcCode", "A123456", case_sensitive=False),
             ])


# EXAMPLE: /InvoiceSections/get/InvoiceSection
@try_manual
def step__invoicesections_get_invoicesection(test):
    test.cmd('az billing invoice section show '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}" '
             '--name "{myInvoiceSection}"',
             checks=[
                 test.check("name", "{myInvoiceSection}", case_sensitive=False),
                 test.check("displayName", "invoiceSection1", case_sensitive=False),
                 test.check("labels.costCategory", "Support", case_sensitive=False),
                 test.check("labels.pcCode", "A123456", case_sensitive=False),
             ])


# EXAMPLE: /InvoiceSections/get/InvoiceSectionsListByBillingProfile
@try_manual
def step__invoicesections_get(test):
    test.cmd('az billing invoice section list '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}"',
             checks=[
                 test.check('length(@)', 1),
             ])


# EXAMPLE: /BillingPermissions/get/BillingAccountPermissionsList
@try_manual
def step__billingpermissions_get3(test):
    test.cmd('az billing permission list '
             '--account-name "{myBillingAccount}"',
             checks=[])


# EXAMPLE: /Policies/put/UpdatePolicy
@try_manual
def step__policies_put_updatepolicy(test):
    test.cmd('az billing policy update '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}" '
             '--marketplace-purchases "OnlyFreeAllowed" '
             '--reservation-purchases "NotAllowed" '
             '--view-charges "Allowed"',
             checks=[])


def step__policies_showpolicybycustomer(test):
    test.cmd('az billing policy show '
             '--account-name "{myBillingAccount}" '
             '--customer-name "{myCustomer}"',
             checks=[])


def step__policies_showpolicybyprofile(test):
    test.cmd('az billing policy show '
             '--account-name {myBillingAccount} '
             '--profile-name {myBillingProfile}',
             checks=[])


# EXAMPLE: /Products/get/Product
@try_manual
def step__products_get_product(test):
    test.cmd('az billing product show '
             '--account-name "{myBillingAccount}" '
             '--name "{myProduct}"',
             checks=[])


# EXAMPLE: /Products/get/ProductsListByBillingAccount
@try_manual
def step__products_get_productslistbybillingaccount(test):
    test.cmd('az billing product list '
             '--account-name "{myBillingAccount}"',
             checks=[])


# EXAMPLE: /Products/get/ProductsListByBillingProfile
@try_manual
def step__products_get_productslistbybillingprofile(test):
    test.cmd('az billing product list '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}"',
             checks=[])


# EXAMPLE: /Products/get/ProductsListByInvoiceSection
@try_manual
def step__products_get_productslistbyinvoicesection(test):
    test.cmd('az billing product list '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}" '
             '--invoice-section-name "{myInvoiceSection}"',
             checks=[])


# EXAMPLE: /Products/get/ProductsListByInvoiceSection
@try_manual
def step__products_get_productslistbyinvoicesection(test):
    test.cmd('az billing product list '
             '--account-name "{myBillingAccount}" '
             '--profile-name "{myBillingProfile}" '
             '--invoice-section-name "{myInvoiceSection}"',
             checks=[])


# EXAMPLE: /Products/patch/UpdateBillingProperty
@try_manual
def step__products_patch_updatebillingproperty(test, checks=None):
    test.cmd('az billing product update '
             '--account-name "{myBillingAccount}" '
             '--auto-renew "Off" '
             '--name "{myProduct}"',
             checks=[
                 test.check("autoRenew", "Off", case_sensitive=False),
                 test.check("name", "{myProduct}", case_sensitive=False),
             ])


# EXAMPLE: /Products/post/MoveProduct
@try_manual
def step__products_post_moveproduct(test):
    test.cmd('az billing product move '
             '--account-name "{myBillingAccount}" '
             '--destination-invoice-section-id "/providers/Microsoft.Billing/billingAccounts/{billingAccountName}/billi'
             'ngProfiles/{billingProfileName}/invoiceSections/{newInvoiceSectionName}" '
             '--name "{myProduct}"',
             checks=[])


# EXAMPLE: /Products/post/SubscriptionMoveValidateFailure
@try_manual
def step__products_post(test):
    test.cmd('az billing product validate-move '
             '--account-name "{myBillingAccount}" '
             '--destination-invoice-section-id "/providers/Microsoft.Billing/billingAccounts/{billingAccountName}/billi'
             'ngProfiles/{billingProfileName}/invoiceSections/{newInvoiceSectionName}" '
             '--name "{myProduct}"',
             checks=[])


# EXAMPLE: /Products/post/SubscriptionMoveValidateSuccess
@try_manual
def step__products_post2(test):
    test.cmd('az billing product validate-move '
             '--account-name "{myBillingAccount}" '
             '--destination-invoice-section-id "/providers/Microsoft.Billing/billingAccounts/{billingAccountName}/billi'
             'ngProfiles/{billingProfileName}/invoiceSections/{newInvoiceSectionName}" '
             '--name "{myProduct}"',
             checks=[])


# EXAMPLE: /Transactions/get/TransactionsListByInvoice
@try_manual
def step__transactions_get_transactionslistbyinvoice(test):
    test.cmd('az billing transaction list '
             '--account-name "{myBillingAccount}" '
             '--invoice-name "{myInvoice}"',
             checks=[])


# Env cleanup
# @try_manual
# def cleanup(test):
#     pass


# Testcase
# @try_manual
# def call_scenario(test):
#     setup(test)
#     step__billingaccounts_get_billingaccounts(test)
#     step__billingaccounts_get_billingaccountslist(test)
#     step__billingaccounts_get(test)
#     step__billingaccounts_get2(test)
#     step__billingaccounts_get_billingaccountwithexpand(test)
#     step__billingaccounts_patch_updatebillingaccount(test)
#     step__agreements_get_agreementbyname(test)
#     step__agreements_get(test)
#     step__billingprofiles_put_createbillingprofile(test)
#     step__billingprofiles_get_billingprofile(test)
#     step__billingprofiles_get(test)
#     step__billingprofiles_get2(test)
#     step__billingprofiles_get_billingprofilewithexpand(test)
#     step__availablebalances_get(test)
#     step__billingroleassignments_get(test)
#     step__billingpermissions_get(test)
#     step__billingroleassignments_get2(test)
#     step__billingroleassignments_get3(test)
#     step__billingroleassignments_delete(test)
#     step__billingroleassignments_delete2(test)
#     step__billingproperty_patch_updatebillingproperty(test)
#     step__customers_get_customerslistbybillingaccount(test)
#     step__customers_get_customerslistbybillingprofile(test)
#     step__customers_get_customerwithexpand(test)
#     step__billingsubscriptions_get_billingsubscription(test)
#     step__billingpermissions_get2(test)
#     step__billingpermissions_get2(test)
#     step__customers_get_customer(test)
#     step__billingproperty_get_billingproperty(test)
#     step__billingsubscriptions_get(test)
#     step__billingsubscriptions_get2(test)
#     step__billingsubscriptions_get3(test)
#     step__billingsubscriptions_get4(test)
#     step__billingsubscriptions_patch(test)
#     step__billingsubscriptions_post(test)
#     step__billingsubscriptions_post2(test)
#     step__billingpermissions_get3(test)
#     step__invoices_get(test)
#     step__invoices_get_billingaccountinvoiceslist(test)
#     step__billingsubscriptions_post3(test)
#     step__invoices_get_creditnote(test)
#     step__invoices_get_invoice(test)
#     step__invoices_get2(test)
#     step__invoices_get_invoiceslistbybillingprofile(test)
#     step__invoices_get3(test)
#     step__invoices_get_invoicewithrebilldetails(test)
#     step__invoices_get_voidinvoice(test)
#     step__invoicesections_put_putinvoicesection(test)
#     step__invoicesections_get_invoicesection(test)
#     step__invoicesections_get(test)
#     step__billingroleassignments_delete3(test)
#     step__policies_put_updatepolicy(test)
#     step__products_get_product(test)
#     step__products_get_productslistbybillingaccount(test)
#     step__products_get_productslistbybillingprofile(test)
#     step__products_get_productslistbyinvoicesection(test)
#     step__products_get_productslistbyinvoicesection(test)
#     step__products_patch_updatebillingproperty(test)
#     step__products_post_moveproduct(test)
#     step__products_post(test)
#     step__products_post2(test)
#     step__transactions_get_transactionslistbyinvoice(test)
#     cleanup(test)


# @try_manual
# class BillingManagementClientScenarioTest(ScenarioTest):

#     def test_billing(self):

#         self.kwargs.update({
#             'myBillingAccount': '{billingAccountName}',
#             'myBillingProfile': '{billingProfileName}',
#             'myCustomer': '{customerName}',
#             'myInvoiceSection': '{invoiceSectionName}',
#             'myProduct': '{productName}',
#             'myInvoice': '{invoiceName}',
#             'myBillingRoleAssignment': '{billingRoleAssignmentName}',
#             'myAgreement': '{agreementName}',
#         })

#         call_scenario(self)
#         calc_coverage(__file__)
#         raise_if()


@record_only()
@try_manual
class BillingAccountScenarioTest(ScenarioTest):
    def test_billing_account_list_and_show(self):

        # list billing accounts without any arguments
        step__billingaccounts_get_billingaccountslist(self)

        # list billing accounts with expanded arguments
        step__billingaccounts_get(self)
        step__billingaccounts_get2(self)

        self.kwargs.update({
            "myBillingAccount": "aff095f4-f26b-5334-db79-29704a77c0e5:8d5301c9-db55-4eb6-8611-9db0417d6cb2_2019-05-31"   # This name is give by service team
        })

        # show billing account with name
        step__billingaccounts_get_billingaccounts(self)
        # show billing account with expaned arguments
        step__billingaccounts_get_billingaccountwithexpand(self)

    def test_billing_account_update(self):
        pass

        # # show
        # step__billingaccounts_get_billingaccountwithexpand(self)

        # # update
        # step__billingaccounts_patch_updatebillingaccount(self)

        # # show and check for updated properties
        # step__billingaccounts_get_billingaccountwithexpand(self)


@record_only()
@try_manual
class BillingProfileScenarioTest(ScenarioTest):

    def setUp(self):
        super().setUp()
        self.kwargs.update({
            "myBillingAccount": "aff095f4-f26b-5334-db79-29704a77c0e5:8d5301c9-db55-4eb6-8611-9db0417d6cb2_2019-05-31"   # This name is give by service team
        })

    def test_biiling_profile_list_and_show(self):

        # list billing profiles under a certain billing account
        step__billingprofiles_get(self)

        # list billing profiles with expanded arguments
        step__billingprofiles_get2(self)

        self.kwargs.update({
            "myBillingProfile": "DN7P-GKOI-BG7-AJ4D-SGB"
        })

        # show a billing profile with default properties
        step__billingprofiles_get_billingprofile(self)
        # show a billing profile with expaned properties
        step__billingprofiles_get_billingprofilewithexpand(self)

    def test_billing_profile_create_and_update(self):

        # list existing billing profiles
        step__billingprofiles_get2(self)

        self.kwargs.update({
            "myBillingProfile": "XXGU-AF7A-BG7-AJ4D-123"
        })

        # create a new billing profile
        # step__billingprofiles_put_createbillingprofile(self)    # this API is not working

        # list new appended billing profiles
        step__billingprofiles_get2(self)

        # show that billing profile
        step__billingprofiles_get_billingprofile(self)


@record_only()
class BillingAccountScenarioTest(ScenarioTest):
    def test_billing_balance_show(self):
        self.kwargs.update({
            # This name is give by service team
            "myBillingAccount": "aff095f4-f26b-5334-db79-29704a77c0e5:8d5301c9-db55-4eb6-8611-9db0417d6cb2_2019-05-31",
            "myBillingProfile": "ROHX-DYIN-BG7-AJ4D-SGB"
        })
        step__availablebalances_get(self)


@record_only()
class BillingCustomerScenarioTest(ScenarioTest):

    def test_billing_customer_list_and_show(self):

        self.kwargs.update({
            # This name is give by service team
            "myBillingAccount": "aff095f4-f26b-5334-db79-29704a77c0e5:8d5301c9-db55-4eb6-8611-9db0417d6cb2_2019-05-31",
            "myBillingProfile": "ROHX-DYIN-BG7-AJ4D-SGB",
        })

        step__customers_get_customerslistbybillingaccount(self)
        step__customers_get_customerslistbybillingprofile(self)

        self.kwargs.update({
            "myCustomer": "ba897bfa-7111-465b-8a03-2729e540ef86"
        })

        step__customers_get_customer(self)
        step__customers_get_customerslistbybillingaccount(self)


@record_only()
class BillingPolicyScenarioTest(ScenarioTest):

    def test_billing_policy_show_and_update(self):
        self.kwargs.update({
            "myBillingAccount": "aff095f4-f26b-5334-db79-29704a77c0e5:8d5301c9-db55-4eb6-8611-9db0417d6cb2_2019-05-31",
            "myBillingProfile": "ROHX-DYIN-BG7-AJ4D-SGB",
            "myCustomer": "ba897bfa-7111-465b-8a03-2729e540ef86"
        })

        step__policies_showpolicybycustomer(self)
        step__policies_showpolicybyprofile(self)

        step__policies_put_updatepolicy(self)

        step__policies_showpolicybycustomer(self)
        step__policies_showpolicybyprofile(self)


@record_only()
class BillingPropertyScenarioTest(ScenarioTest):

    def test_billing_property_show(self):
        # have to set a subscription first
        # removed sensitive information manually from recording file
        step__billingproperty_get_billingproperty(self)


# All of operations are forbidden
@record_only()
class BillingInvoiceScenarioTest(ScenarioTest):
    def test_billing_invoice_list_and_show(self):
        self.kwargs.update({
            "myBillingAccount": "aff095f4-f26b-5334-db79-29704a77c0e5:8d5301c9-db55-4eb6-8611-9db0417d6cb2_2019-05-31",
        })

        # list by billing account
        # step__invoices_get_billingaccountinvoiceslist(self)
        # step__invoices_get2(self)
        # list by billing account and billing profile
        # step__invoices_get_invoiceslistbybillingprofile(self)


@record_only()
class BillingInvoiceSectionScenarioTest(ScenarioTest):
    def setUp(self):
        super().setUp()

        self.kwargs.update({
            "myBillingAccount": "aff095f4-f26b-5334-db79-29704a77c0e5:8d5301c9-db55-4eb6-8611-9db0417d6cb2_2019-05-31",
            "myBillingProfile": "ROHX-DYIN-BG7-AJ4D-SGB",
            "myInvoiceSection": "7S7S-YONO-PJA-AJ4D-SGB"
        })

    def test_billing_invoice_section_list_and_show(self):
        step__invoicesections_get(self)
        step__invoicesections_get_invoicesection(self)

    def test_billing_section_create(self):
        self.kwargs.update({
            "myInvoiceSection": "7S7S-YONO-PJA-AJ4D-123"
        })
        step__invoicesections_put_putinvoicesection(self)
        step__invoicesections_get(self)
        step__invoicesections_get_invoicesection(self)


# class BillingTransactionScenarioTest(ScenarioTest):
#     def test_billing_transaction_list(self):
#         self.kwargs.update({
#             "myBillingAccount": "aff095f4-f26b-5334-db79-29704a77c0e5:8d5301c9-db55-4eb6-8611-9db0417d6cb2_2019-05-31",
#             "myInvoice": ""
#         })
#         step__transactions_get_transactionslistbyinvoice(self)


@record_only()
class BillingAgreementScenarioTest(ScenarioTest):
    def test_agreement_list_and_show(self):
        self.kwargs.update({
            "myBillingAccount": "db038d21-b0d2-463c-942f-b09127c6f4e4:7c9c4a38-593e-479e-8958-9a338a0d8d02_2019-05-31",
            "myAgreement": "NotFountResource"
        })

        step__agreements_get(self)
        step__agreements_get_agreementbyname(self)


@record_only()
class BillingPermissionScenarioTest(ScenarioTest):

    def test_permission_list(self):
        self.kwargs.update({
            "myBillingAccount": "db038d21-b0d2-463c-942f-b09127c6f4e4:7c9c4a38-593e-479e-8958-9a338a0d8d02_2019-05-31",
            "myBillingProfile": "2A3E-KXSS-BG7-TGB",
            "myInvoiceSection": "CGFY-TOZ5-PJA-TGB"
        })

        # list by invoice name
        step__billingpermissions_get(self)

        # list by customer name
        # step__billingpermissions_get2(self)   # no customer to list and it will raise exception

        # # list by billing account
        step__billingpermissions_get3(self)


@record_only()
class BillingRoleAssignmentScenarioTest(ScenarioTest):

    def setUp(self):
        super().setUp()

        self.kwargs.update({
            "myBillingAccount": "db038d21-b0d2-463c-942f-b09127c6f4e4:7c9c4a38-593e-479e-8958-9a338a0d8d02_2019-05-31",
            "myBillingProfile": "ZY2R-LX3G-BG7-TGB",
            "myInvoiceSection": "ES5C-GD7Y-PJA-TGB"
        })

    def test_role_assignment_list_and_show(self):
        # list by billing account
        step__billingroleassignments_get2(self)

        # list by billing profile
        step__billingroleassignments_get3(self)

        # list by invoice name
        step__billingroleassignments_get(self)


@record_only()
class BillingRoleDefinitionScenarionTest(ScenarioTest):
    def test_role_definition_list_and_show(self):
        self.kwargs.update({
            "myBillingAccount": "db038d21-b0d2-463c-942f-b09127c6f4e4:7c9c4a38-593e-479e-8958-9a338a0d8d02_2019-05-31",
            "myBillingProfile": "ZY2R-LX3G-BG7-TGB",
            "myInvoiceSection": "S5C-GD7Y-PJA-TGB",
        })

        # list by billing account
        step__billingroledefinitions_get(self)
        # list by billing profile
        step__billingroledefinitions_get2(self)
        # list by billing invoice section
        step__billingroledefinitions_get3(self)

        self.kwargs.update({
            "roleDefinitionName": "50000000-aaaa-bbbb-cccc-100000000000"
        })

        # show by billing profile
        self.cmd('az billing role-definition show '
                 '--account-name "{myBillingAccount}" '
                 '--name {roleDefinitionName} ',
                 checks=[])

        self.kwargs.update({
            "roleDefinitionName": "40000000-aaaa-bbbb-cccc-100000000000"
        })

        # show by billing profile
        self.cmd('az billing role-definition show '
                 '--account-name "{myBillingAccount}" '
                 '--name {roleDefinitionName} '
                 '--profile-name "{myBillingProfile}" ',
                 checks=[])

        self.kwargs.update({
            "roleDefinitionName": "30000000-aaaa-bbbb-cccc-100000000000"
        })

        # show by invoice section and role definition name
        self.cmd('az billing role-definition show '
                 '--account-name "{myBillingAccount}" '
                 '--name {roleDefinitionName} '
                 '--profile-name "{myBillingProfile}" '
                 '--invoice-section-name "{myInvoiceSection}"',
                 checks=[])


@record_only()
class BillingInstructionScenarioTest(ScenarioTest):
    def setUp(self):
        super().setUp()

        self.kwargs.update({
            "myBillingAccount": "db038d21-b0d2-463c-942f-b09127c6f4e4:7c9c4a38-593e-479e-8958-9a338a0d8d02_2019-05-31",
            "myBillingProfile": "ZY2R-LX3G-BG7-TGB",
            "myInstruction": "404Instruction",
        })

    def test_instruction_list_and_show(self):
        step__instructions_get(self)

        step__instructions_get_instruction(self)

    def test_instruction_create_and_update(self):

        self.kwargs.update({
            "myInstruction": "inst-1",
        })

        step__instructions_put_putinstruction(self)

        self.cmd('az billing instruction update '
                 '--account-name "{myBillingAccount}" '
                 '--profile-name "{myBillingProfile}" '
                 '--name "{myInstruction}" '
                 '--amount 12')

        step__instructions_get_instruction(self)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azure_devtools.scenario_tests import AllowLargeResponse
from azure.cli.core.local_context import AzCLILocalContext, ALL, LOCAL_CONTEXT_FILE
from azure.cli.core.util import CLIError
from azure.cli.testsdk import (
    JMESPathCheck,
    NoneCheck,
    ResourceGroupPreparer,
    StringContainCheck,
    ScenarioTest,
    LocalContextScenarioTest)

# Constants
SERVER_NAME_PREFIX = 'azuredbclitest-'
SERVER_NAME_MAX_LENGTH = 20


class FlexibleServerLocalContextScenarioTest(LocalContextScenarioTest):

    postgres_location = 'eastus2euap'
    mysql_location = 'eastus2euap'

    @AllowLargeResponse()
    @ResourceGroupPreparer(location=postgres_location)
    def test_postgres_flexible_server_local_context(self, resource_group):
        self._test_flexible_server_local_context('postgres', resource_group)

    @AllowLargeResponse()
    @ResourceGroupPreparer(location=mysql_location)
    def test_mysql_flexible_server_local_context(self, resource_group):
        self._test_flexible_server_local_context('mysql', resource_group)

    def _test_flexible_server_local_context(self, database_engine, resource_group):
        self.cmd('config param-persist on')
        if database_engine == 'mysql':
            location = self.mysql_location
        else:
            location = self.postgres_location

        server_name = self.create_random_name(SERVER_NAME_PREFIX, SERVER_NAME_MAX_LENGTH)

        self.cli_ctx.local_context.set(['all'], 'resource_group_name', resource_group)
        self.cli_ctx.local_context.set(['all'], 'location', location)

        self.cmd('{} flexible-server create -n {} --public-access none'.format(database_engine, server_name))

        local_context_info = self.cmd('config param-persist show').get_output_in_json()

        self.cmd('{} flexible-server show'.format(database_engine),
                 checks=[JMESPathCheck('resourceGroup', local_context_info['all']['resource_group_name']),
                         JMESPathCheck('location', 'East US 2 EUAP'),
                         JMESPathCheck('name', local_context_info[database_engine + ' flexible-server']['server_name']),
                         JMESPathCheck('administratorLogin', local_context_info[database_engine + ' flexible-server']['administrator_login'])])

        self.cmd('{} flexible-server show-connection-string'.format(database_engine),
                 checks=[StringContainCheck(local_context_info[database_engine + ' flexible-server']['administrator_login'])]).get_output_in_json()

        self.cmd('{} flexible-server list-skus'.format(database_engine))

        self.cmd('{} flexible-server delete --yes'.format(database_engine))

        delete_local_context_info = self.cmd('config param-persist show').get_output_in_json()
        self.assertNotIn(database_engine + ' flexible-server', delete_local_context_info)
        self.assertNotIn(local_context_info[database_engine + ' flexible-server']['server_name'], delete_local_context_info)
        self.assertNotIn(local_context_info[database_engine + ' flexible-server']['administrator_login'], delete_local_context_info)



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# common utilities for scripts

from __future__ import print_function


def get_repo_root():
    """
    Returns the root path to this repository. The root is where .git folder is.
    """
    import os.path
    here = os.path.dirname(os.path.realpath(__file__))

    while not os.path.exists(os.path.join(here, '.git')):
        here = os.path.dirname(here)

    return here


if __name__ == '__main__':
    print(get_repo_root())

